package no.ntnu.stockmeks;

import no.ntnu.stockmeks.agent.Agent;
import no.ntnu.stockmeks.agent.AgentAdministrator;

import java.util.ArrayList;
import java.io.*;


/*
 * Class for evaluation of agents' perfomance
 */
public class AgentEvaluator {
	
	Logger log = new Logger("Evaluator");
	int sellSignal = Agent.SIGNAL_SELL;
	int buySignal = Agent.SIGNAL_BUY;
	
	ArrayList<Integer> agentSingleWinningTransactions;
	ArrayList<Integer> agentSingleLosingTransactions;
	ArrayList<Double> agentSingleEarnings;
	ArrayList<Integer> agentBuys;
	ArrayList<Integer> agentSells;
	ArrayList<Double> agentMoney;
	ArrayList<Double> agentYield;
	ArrayList<Integer> agentCountingTickers;
	
	ArrayList<Double> monthEarnings;
	ArrayList<Double> monthBH;
	ArrayList<Integer> monthStocks;
	
	AgentAdministrator[][] aa;
	
	StockTicker[] tickers;
	Agent[] agents;
	
	int startDay = 30;

	
	int startAfterYear = 1998;
	int startAfterMonth = 12;
	int startAfterDay = 31;
	
	int endBeforeYear = 2007;
	int endBeforeMonth = 1;
	int endBeforeDay = 1;
	
	
	
	
	public AgentEvaluator(StockTicker[] tickers,Agent[] agents){
		agentSingleWinningTransactions = new ArrayList<Integer>();
		agentSingleLosingTransactions = new ArrayList<Integer>();
		agentSingleEarnings = new ArrayList<Double>();
		agentBuys = new ArrayList<Integer>();
		agentSells = new ArrayList<Integer>();
		agentMoney = new ArrayList<Double>();
		agentYield = new ArrayList<Double>();
		agentCountingTickers = new ArrayList<Integer>();
		
		monthEarnings = new ArrayList<Double>();
		monthBH = new ArrayList<Double>();
		monthStocks = new ArrayList<Integer>();

		
		initialize(tickers,agents);
	}
	
	
	public void initialize(StockTicker[] tickers, Agent[] agents){
		aa = new AgentAdministrator[tickers.length][agents.length];
		
		for(int i=0;i<tickers.length;i++){
			for(int j=0;j<agents.length;j++){
				aa[i][j] = new AgentAdministrator(tickers[i],agents[j]);
			}
		}
		
		this.agents = agents;
		this.tickers = tickers;
		
		
	}
	

	/*
	 * Starting monthly evuation
	 */
	public void evaluateMonths(int fromYear, int toYear){
		
		log.debug("Agent: " + agents[0].getAgentName());
		
		
		for(int year=fromYear;year<=toYear;year++){
			endBeforeYear = year;
			for(int i=2;i<14;i++){
				endBeforeMonth = i;
				log.debug((i-1) + " " + year +": " + evaluate(false,true,true,true,0) + "%");
			}
		}
		
		String[] monthNames = {"jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"};
		
		log.debug("#Results: #");
		for(int i=0;i<monthEarnings.size();i++){
			int inYear = (int)Math.floor(i/12);
			int inMonth = (i) - (inYear*12);
			log.output(monthNames[inMonth] + " " + (fromYear+inYear)+" & " + monthBH.get(i) + "\\% & " + monthEarnings.get(i) +"\\% & " + monthStocks.get(i)+ "\\\\");
		}
		
		
		
	}
	

	
	public double evaluate(boolean useExtremes, boolean printSummary,int agentNumber){
		return evaluate(useExtremes,true,false,printSummary,agentNumber);
	}
	
	
	
	
	public void clearAgentInfo(){
		agentSingleWinningTransactions.clear();
		agentSingleLosingTransactions.clear();
		agentSingleEarnings.clear();
		agentBuys.clear();
		agentSells.clear();
		agentMoney.clear();
		agentYield.clear();
		agentCountingTickers.clear();
		
		monthBH.clear();
		monthEarnings.clear();
		monthStocks.clear();
	}
	
	
	public double evaluate(boolean useExtremes, boolean doLimitDays, boolean doReturnAllInMoney, boolean printSummary,int agentNumber){
		//evaluating
		clearAgentInfo();
		double allInMoney = 0.0;
		
		
		try{
			FileWriter fw = new FileWriter("transactionsummary.txt");
			BufferedWriter bw = new BufferedWriter(fw);

			
			FileWriter fw2;
			if(doLimitDays) fw2 = new FileWriter("monthly//"+endBeforeYear+"-"+(endBeforeMonth-1)+"-"+endBeforeDay+"_evaluation-table.txt"); 
			else fw2 = new FileWriter("evaluation-table.txt");
			BufferedWriter bw2 = new BufferedWriter(fw2);
			
			String[] stockSummary = new String[agents.length];

			
			double[] buyAndHoldYield = new double[tickers.length];
			double[] sellAndWaitYield = new double[tickers.length];
			

			if(printSummary){
				bw.write("Buy and hold yield: ");
				double totalYield = 0.0;
				int numCounting = 0;
				int profitableTrades = 0;
				for(int i=0;i<tickers.length;i++){
					int theStartDay = startDay;
					int theEndDay = tickers[i].getSize()-1;
					if(doLimitDays){
						theStartDay = tickers[i].getStartDayNumber(startDay,startAfterDay,startAfterMonth,startAfterYear);
						theEndDay = tickers[i].getEndDayNumber(endBeforeDay,endBeforeMonth,endBeforeYear);
					}
					
					
					if(theStartDay>0 && theEndDay>0){
						numCounting++;
						double tickerStartPrice = tickers[i].getClose(theStartDay);
						double tickerEndPrice = tickers[i].getClose(theEndDay);
						double buyAndHoldEarnings = (tickerEndPrice-tickerStartPrice) / tickerStartPrice;
						
						if(buyAndHoldEarnings>0) profitableTrades++;
						totalYield += buyAndHoldEarnings;
						
						bw.write("\n" + tickers[i].getTickerName().toUpperCase() + ": " + buyAndHoldEarnings);
					}
				}
				double mediumTotalYield = (double)Math.round(totalYield/((double)numCounting)*1000.0)/10.0;
				bw.write("\nTotal yield: " + totalYield + ", (" + mediumTotalYield+" in avg over " + numCounting+" trades)\n");
				bw2.write("Buy and hold & "+(mediumTotalYield) + "\\% & "+(((double)profitableTrades/(double)numCounting)*100.0)+ " (of " + numCounting + ") & " + (mediumTotalYield) + "\\% \\\\ \n");
				monthBH.add((mediumTotalYield));
				monthStocks.add(numCounting);
				bw.flush();
				bw2.flush();
			}
			
			for(int i=0;i<agents.length;i++){

				agentSingleWinningTransactions.add(0);
				agentSingleLosingTransactions.add(0);
				agentSingleEarnings.add(0.0);
				agentBuys.add(0);
				agentSells.add(0);
				agentMoney.add(0.0);
				agentYield.add(0.0);
				agentCountingTickers.add(0);
				
				stockSummary[i] = "";
				
				
				for(int j=0;j<tickers.length;j++){
					if(i==0){
						double tickerStartPrice = tickers[j].getClose(i);
						double tickerEndPrice = tickers[j].getClose(tickers[j].getSize()-1);
						buyAndHoldYield[j] = (-1.00) + tickerEndPrice / tickerStartPrice;
						sellAndWaitYield[j] =  (-1.00) + tickerStartPrice / tickerEndPrice;
					}

					 stockSummary[i] += evaluateWithoutExtremes(aa[j][i],i,doLimitDays) + "\n";
					 
				}
			}
			
			double allEarnings = 0.0;
			int allWinningTransactions = 0;
			int allLosingTransactions = 0;
			int allBuys = 0;
			int allSells = 0;
			double allMoney = 0.0;
			
			for(int i=0;i<agents.length;i++){

				if(printSummary){
					bw.write("\nAgent " + agents[i].getAgentName());
					double mediumTotalEarnings = agentSingleEarnings.get(i)/(agentSingleWinningTransactions.get(i)+agentSingleLosingTransactions.get(i));
					double mediumAgentYield = agentSingleEarnings.get(i)/(double)agentCountingTickers.get(i);
					
					bw.write("\nTotal earnings: " + agentSingleEarnings.get(i) + "\nMedium earnings:  " + mediumTotalEarnings + "\n"+agentSingleWinningTransactions.get(i)+" winning transactions \n" + agentSingleLosingTransactions.get(i)+" losing transactions");
					bw.write("\n" + agentBuys.get(i) + " buys and " + agentSells.get(i) + " sells");
					bw.write("\n-----");
					bw.write("\n"+stockSummary[i]);
					
					double agentSumYield = (double)(Math.round(mediumAgentYield*1000.0))/10.0;
					double agentSumCorrect = (double)(Math.round(1000.0*((double)agentSingleWinningTransactions.get(i)/(double)(agentSingleLosingTransactions.get(i)+agentSingleWinningTransactions.get(i)))))/10.0;
					double agentSumAllinYield = Math.round(10.0*((agentMoney.get(i)/agentCountingTickers.get(i))-100.0))/10.0;
					
					
					log.debug("agentMoney e: " + agentMoney.get(i) + ", counting tickers: "+ agentCountingTickers.get(i));
					
					bw2.write(agents[i].getAgentName() + " & " + agentSumYield + "\\% & " + agentSumCorrect + "\\% (of " + (agentSingleLosingTransactions.get(i)+agentSingleWinningTransactions.get(i)) +") & " + agentSumAllinYield + "\\% \\\\ \n");
					monthEarnings.add(agentSumAllinYield);
					//bw2.write("Buy and hold & "+(mediumTotalYield*100.0) + "\\% & "+(((double)profitableTrades/(double)numCounting)*100.0)+ " (of " + numCounting + ") & " + (mediumTotalYield*100.0) + "\\%");
				}
				else if(doReturnAllInMoney){
					double agentSumAllinYield = Math.round(10.0*((agentMoney.get(i)/agentCountingTickers.get(i))-100.0))/10.0;
					allInMoney = agentSumAllinYield;
				}
				
				allEarnings += agentSingleEarnings.get(i);
				allWinningTransactions += agentSingleWinningTransactions.get(i);
				allLosingTransactions += agentSingleLosingTransactions.get(i);
				allBuys += agentBuys.get(i);
				allSells += agentSells.get(i);
				allMoney += agentMoney.get(i);
				
			}
			
			
			
			int allTransactions = allWinningTransactions + allLosingTransactions;
			double winPercent = (double)allWinningTransactions/(double)(allTransactions);
			double mediumYield = allEarnings/(double)allTransactions;
			double avgRoi = allEarnings/(double)(agents.length);

			double moneyAvg = allMoney/((double)tickers.length);

			if(printSummary){
				bw.write("\n%% Summary %%");
				bw.write("\nTotal earnings: " + allEarnings);
				bw.write("\nMedium earnings: " + mediumYield);
				bw.write("\nNof transaction: " + allTransactions);
				bw.write("\nWinning transaction: " + allWinningTransactions);
				bw.write("\nLosing transactions: " + allLosingTransactions);
				bw.write("\nNof buys: " + allBuys);
				bw.write("\nNof sells: " + allSells);
				bw.write("\nWin percent: " + winPercent);
				bw.write("\nAverage return on investement: " + avgRoi);
				bw.write("\nMoney left: " + allMoney);
				bw.write("\nMoney avg left: " + moneyAvg);
			}
			
			
			
			
			
			bw2.close();
			fw2.close();
			
			
			
			bw.close();
			fw.close();
			if(doReturnAllInMoney) return allInMoney;
			else return allEarnings;
		}
		catch(Exception e){
			e.printStackTrace();
			return 0.0;
		}
		
		
	}
	
	public String evaluateWithoutExtremes(AgentAdministrator administrator, int agentNumber,boolean doLimitDays){
		double boughtAt = 0;
		double soldAt = 0;
		
		double money = 100.0;
		double stocksHeld = 0.0;
		double finalMoney = 0.0;
		
		StockTicker ticker = administrator.getTicker();
		
		
		boolean doShortSell = true;
		boolean doPrintTransactions = true;
		
		int winningTransactions = 0;
		int losingTransactions = 0;
		double totalEarnings = 0;
		
		int buys = 0;
		int sells = 0;
		
		
		int limitedStartDay;
		int limitedEndDay;
		if(doLimitDays){
			limitedStartDay = ticker.getStartDayNumber(startDay,startAfterDay,startAfterMonth,startAfterYear);
			limitedEndDay = ticker.getEndDayNumber(endBeforeDay,endBeforeMonth,endBeforeYear);
		}
		else{
			limitedStartDay = startDay;
			limitedEndDay = ticker.getSize()-1;
		}
		
		if(limitedStartDay>0 && limitedEndDay>0 && limitedStartDay<limitedEndDay){
			
			try{
				
				FileWriter fw = new FileWriter("transactionlogs/" + agents[agentNumber].getAgentName().replace(" ","") + "-"+ ticker.getTickerName() + ".txt");
				BufferedWriter bw = new BufferedWriter(fw);

				
				FileWriter fwT = new FileWriter("alltransactions.txt",true);
				BufferedWriter bwT = new BufferedWriter(fwT);
				
				if(doPrintTransactions){
					bw.write("Transaction log for stock " + ticker.getTickerName() + ":\n");
					
				}
				
				
				
				for(int i=limitedStartDay;i<=limitedEndDay;i++){

					
					int signal = administrator.getAgentSignal(i);
					if(boughtAt==0 && signal==buySignal){
						//want to buy
						buys++;
						if(soldAt==0.0){
							boughtAt = ticker.getClose(i);
							if(doPrintTransactions) bw.write(ticker.getDateString(i) + " BUY: "  + ticker.getClose(i) + "\n");
							stocksHeld = (money/boughtAt);
							money = 0.0;
						}
						else{
							double earnings = 1.0-((ticker.getClose(i))/soldAt);
							if(earnings>0) winningTransactions++;
							else losingTransactions++;
							
							bwT.write(earnings +"\n");
							
							totalEarnings+=earnings;

							
							if(doShortSell){
								double spending = stocksHeld*ticker.getClose(i);
								money -= spending;
								soldAt=0.0;
							}
							
							if(money<0){
								finalMoney = money;
								money = 0.0;
							}
							
							if(doPrintTransactions) bw.write(ticker.getDateString(i) + " BUY: "  + ticker.getClose(i) + " [" + (earnings*100.00) + "%] (" + money +")\n");
							
							
							boughtAt = ticker.getClose(i);
							stocksHeld = money/(ticker.getClose(i));
							money = 0.0;
						}
						
					}
					else if(soldAt==0 && signal==sellSignal){
						if(boughtAt==0 && doShortSell){
							//short selling
							sells++;

							soldAt = ticker.getClose(i);
							
							if(doPrintTransactions) bw.write(ticker.getDateString(i) + " SELL: "  + ticker.getClose(i) + "\n");
							stocksHeld = money/soldAt;
							money += money;
							

						}
						else if(boughtAt>0){
							sells++;
							double earnings = (ticker.getClose(i)-boughtAt)/boughtAt;
							if(earnings>0) winningTransactions++;
							else losingTransactions++;
							
							bwT.write(earnings +"\n");

							totalEarnings+=earnings;
							boughtAt = 0;
						
							

							double getting = stocksHeld*ticker.getClose(i);
							money += getting;
							stocksHeld = 0;

							
							if(doPrintTransactions) bw.write(ticker.getDateString(i) + " SELL: "  + ticker.getClose(i) + " [" + (earnings*100) + "%] (" + money +")\n");

							if(doShortSell){
								soldAt = ticker.getClose(i);
								stocksHeld = (money/soldAt);
								money += money;
							}

						}
						
						
					}
				}
				

				if(soldAt>0){
					double earnings = 1.0-((ticker.getClose(limitedEndDay))/soldAt);

					if(earnings>0) winningTransactions++;
					else losingTransactions++;

					bwT.write(earnings +"\n");

					totalEarnings+=earnings;
					double spending = stocksHeld*ticker.getClose(limitedEndDay);
					money -= spending;

					if(doPrintTransactions) bw.write(ticker.getDateString(limitedEndDay) + " BUY: "  + ticker.getClose(limitedEndDay) + " [" + (earnings*100.00) + "%] (" + money +")\n");
				}
				else if(boughtAt>0){
					double earnings = (ticker.getClose(limitedEndDay)-boughtAt)/boughtAt;
					if(earnings>0) winningTransactions++;
					else losingTransactions++;
					
					bwT.write(earnings +"\n");


					totalEarnings+=earnings;
					double getting = stocksHeld*ticker.getClose(limitedEndDay);
					money += getting;

					if(doPrintTransactions) bw.write(ticker.getDateString(limitedEndDay) + " SELL: "  + ticker.getClose(limitedEndDay) + " [" + (earnings*100.00) + "%] (" + money +")\n");
				}
			

				if(money>0){
					finalMoney = money;
				}
				

				int newWinningTransactions = agentSingleWinningTransactions.get(agentNumber) + winningTransactions;
				int newLosingTransactions = agentSingleLosingTransactions.get(agentNumber) + losingTransactions;
				double newEarnings = agentSingleEarnings.get(agentNumber)+totalEarnings;
				int newBuys = agentBuys.get(agentNumber)+buys;
				int newSells = agentBuys.get(agentNumber)+sells;
				double newMoney = agentMoney.get(agentNumber)+finalMoney;
				int agentCounting = agentCountingTickers.get(agentNumber);
				
				int totalTransactions = winningTransactions+losingTransactions;
				double percentWinning = ((double)winningTransactions)/((double)totalTransactions);
				
				if(totalTransactions>0){
					agentCounting++;
					agentMoney.set(agentNumber,newMoney);
				}
				
				agentCountingTickers.set(agentNumber,agentCounting);
				agentSingleWinningTransactions.set(agentNumber,newWinningTransactions);
				agentSingleLosingTransactions.set(agentNumber,newLosingTransactions);
				agentSingleEarnings.set(agentNumber,newEarnings);
				
				agentBuys.set(agentNumber,newBuys);
				agentSells.set(agentNumber,newSells);
				

				
				double mediumEarnings = totalEarnings/((double)(winningTransactions+losingTransactions));
				
				double buyAndHoldYield = (ticker.getClose(limitedEndDay)-ticker.getClose(limitedStartDay))/(ticker.getClose(limitedStartDay));
				if(doPrintTransactions){
					bw.write("\nBuy and hold yield: " + (buyAndHoldYield*100) + "%");
					bw.write("\n\n");
					bw.write(winningTransactions + " winning ("+ (percentWinning*100.0)+ "%) and " + losingTransactions + " losing transactions\n");
					bw.write((totalEarnings*100.0) + "% yield in total\n");
					bw.write((mediumEarnings*100.0) + "% yield per transaction in medium\n");
					bw.write("100 kr has turned into " + finalMoney+" ("+(finalMoney-100.0)+"% earned)\n");
				}
				bwT.close();
				fwT.close();
				
				
				bw.close();
				fw.close();

				if(totalEarnings!=0.0){
					return "" + ticker.getTickerName().toUpperCase() + ": Total: " + totalEarnings + ", medium : " + mediumEarnings + " ("+ winningTransactions + "/" +losingTransactions + ")";
				}
				else return "";
			
			}
			catch(Exception e){
				e.printStackTrace();
				return "";
			}
			
			
			
		}
		else return "";
	}
	
		
		
	
	
}
