package no.ntnu.stockmeks;
import no.ntnu.stockmeks.agent.*;

import org.jgap.*;
//import org.jgap.data.*;
import org.jgap.impl.*;
import java.util.ArrayList;
import java.io.BufferedWriter;
import java.io.FileWriter;

/*
 * Main class for genetic evolution
 */
public class GeneticMain {
	private int MAX_ALLOWED_EVOLUTIONS = 20;
	private int POPULATION_SIZE = 10;
	
	Logger log = new Logger("Genetics");
	ArrayList<Agent> agents;
	
	GeneticFitness fitness;
	int mode;

	public GeneticMain(){
		
	}

	public GeneticMain(int population,int evolutions,int mode){
		MAX_ALLOWED_EVOLUTIONS = evolutions;
		POPULATION_SIZE = population;
		this.mode = mode;
	}
	
	
	 
	public void findSuperAgent(StockTicker[] tickers, Agent[] agents) throws Exception{
		
		log.debug("Starting genetics for population " + POPULATION_SIZE + ", max evoluions: " + MAX_ALLOWED_EVOLUTIONS + "....");
		
		Configuration conf = new DefaultConfiguration();
		conf.setPreservFittestIndividual(true);
		
		FitnessFunction fit = new GeneticFitness(tickers,agents,mode);
		
		conf.setFitnessFunction(fit);
		
		Gene[] agentGenes = new Gene[agents.length];
		
		for(int i=0;i<agentGenes.length;i++){
			agentGenes[i] = new IntegerGene(conf,-10,10);
		}

		IChromosome agentChromosome = new Chromosome(conf,agentGenes);
		conf.setSampleChromosome(agentChromosome);
		conf.setPopulationSize(POPULATION_SIZE);
		Genotype population = Genotype.randomInitialGenotype(conf);

		try{
			FileWriter fw = new FileWriter("genetics.txt");
			BufferedWriter bw = new BufferedWriter(fw);
			String introText = "";
			if(mode==1) introText += "Genetics (using extremes) for population of ";
			else if(mode==2) introText += "Genetics (using correct days) for population of ";
			else introText += "Genetics (normal) for population of ";
			
			bw.write(introText + POPULATION_SIZE + ", max evoluions: " + MAX_ALLOWED_EVOLUTIONS + "\n\n");
			
			
			String agentNames = "";
			for(int i=0;i<agents.length;i++){
				agentNames += (i+1) + ": " + agents[i].getAgentName() + "\n";
			}
			bw.write(agentNames + "\n");
			bw.close();
			fw.close();
			

			
			for(int i=0;i<MAX_ALLOWED_EVOLUTIONS;i++){
				population.evolve();
				IChromosome bestSolution = population.getFittestChromosome();
				String output = (i+1) + " [" + bestSolution.getFitnessValue()  +"] ";

				for(int j=0;j<bestSolution.size();j++){
					output += (Integer)bestSolution.getGene(j).getAllele() +" ";
				}

				System.out.println(output);

				fw = new FileWriter("genetics.txt",true);
				bw = new BufferedWriter(fw);
				bw.write(output + "\n");
				bw.close();
				fw.close();
			}
			
			
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
		
		
		
	}
	

	
}
