package no.ntnu.stockmeks;

import javax.swing.*;
import java.awt.*;
import java.awt.font.TextLayout;
import java.awt.geom.*;
import java.lang.Math;

import no.ntnu.stockmeks.agent.*;
/*
 * Gui class of the program
 */
public class StockChart extends JPanel {
	
	
	StockTicker ticker;
	int chartW;
	int chartH;
	Logger log = new Logger("StockChart");
	
	boolean showAgentAsLines = false;
	
	boolean showAgent = false;
	Agent agentToShow;
	
	
	public int numDays = 3000;
	
	private static final long serialVersionUID = 1L;


	public StockChart(StockTicker ticker){
		this.ticker = ticker;
	}
	
	
	public void paintComponent(Graphics gIn){

       Graphics2D g2 = (Graphics2D)gIn;
	   g2.setColor(new Color(0,0,0));
	   Dimension d = getSize();
	   int chartXSpace = 20;
	   int chartYSpace = 20;
	   
	   if(numDays>ticker.getSize()) numDays = ticker.getSize();
	   log.debug("satt numDays til: " + numDays);
	   int[] signals;
	   if(showAgent){
		    signals = agentToShow.getSignalDays(ticker);
	   }
	   else signals = new int[ticker.getSize()];
	   
	   
	   double lowest = ticker.getLowest(ticker.getSize()-numDays);
	   double highest = ticker.getHighest(ticker.getSize()-numDays);

	   
	   chartW = d.width - chartXSpace;
	   chartH = d.height - chartYSpace;
	   
	   
    	   double chartRange;    	   
    	   if(lowest>0) chartRange = (double)((chartH-(chartYSpace))/(highest-lowest));
    	   else chartRange = 1;
    	   
    	   double dayLength;
    	   if(numDays>0){
    		   dayLength = (chartW-chartXSpace)/((double)numDays); ;
    	   }
    	   else dayLength = (chartW-chartXSpace)/((double)ticker.getSize()); 
    	   double dayCoordinate = chartXSpace;
    	   
    	   log.debug("Chartrange: " + chartRange);
    	   log.debug("Lowest siste " + numDays + " dager er: " + lowest);
    	   log.debug("highest siste " + numDays + " dager er: " + highest);
    	   log.debug("Fusste dag koordimeks: " + dayCoordinate);
    	   
    	   //values on y-axis
    	   int pixYInc = 30;
	   	    for(int i=0;(i*pixYInc)<chartH;i++){
	   	    	long textAtYCoordinate = Math.round(lowest + (i*pixYInc/chartRange));
	   	    	TextLayout txt = new TextLayout(""+textAtYCoordinate, new Font("Helvetica", 0, 10), g2.getFontRenderContext());
			 	txt.draw(g2, chartW, (float)Math.round(chartH-(chartYSpace) - (i*pixYInc)));
	   	    }

    	   //values at x-axis
	   	    int pixXInc = 100;
	   	    for(int i=0;(i*pixXInc)<chartW;i++){
	   	    	String tickDate = ticker.getDateString((int)(Math.round((i*pixXInc)/dayLength)),numDays);
	   	    	TextLayout txt = new TextLayout(tickDate, new Font("Helvetica", 0, 10), g2.getFontRenderContext());
			 	txt.draw(g2, (i*pixXInc), chartH+10);
	   	    }
	   	    
	   	    
	   	    
    	   for(int i=ticker.getSize()-numDays;i<ticker.getSize()-1;i++){
    		   double firstCoordinateY = chartH - ((ticker.getClose(i)-lowest)*chartRange);
    		   double nextCoordinateY = chartH - ((ticker.getClose(i+1)-lowest)*chartRange);
       		   double nextCoordinateX = dayCoordinate + dayLength;
       		    		   
    		   
       		   if(showAgent && signals[i]==-1) g2.setColor(new Color(255,0,0));
       		   else if(showAgent && signals[i]==1) g2.setColor(new Color(0,0,255));
       		   else g2.setColor(new Color(0,0,0));
       		   
    		   g2.draw(new Line2D.Double(dayCoordinate,firstCoordinateY,nextCoordinateX,nextCoordinateY));
    		   
    		   if(showAgent && !showAgentAsLines){
    			   if(signals[i]==-1){
    				   //shape indicating sell signal
    				   Shape meks = new Rectangle2D.Double(dayCoordinate-5,firstCoordinateY-5,10,10);

    				   g2.setColor(new Color(255,0,0));
    				   g2.fill(meks);
    				   g2.draw(meks);
    				   g2.setColor(new Color(0,0,0));
    				   
    				   
    				   
    			   }
    			   else if(signals[i]==1){
    				   //shape indicating buy signal
    				   Shape meks = new Ellipse2D.Double(dayCoordinate-5,firstCoordinateY-5,10,10);
    				   g2.setColor(new Color(0,0,255));
    				   g2.draw(meks);

    				   g2.setColor(new Color(0,0,0));
    			   }
    			   
    		   }
    		   dayCoordinate = nextCoordinateX;
    	   }
	}
	
	public void setTicker(StockTicker ticker){
		this.ticker = ticker;
	}
	
	public void hideAgent(){
		showAgent = false;
	}
	
	public void setAgent(Agent agent){
		this.agentToShow = agent;
		showAgent = true;
	}
	
}
