package no.ntnu.stockmeks;

import java.io.*;
import java.util.Calendar;
import java.net.*;

public class StockHistoryImporter {
	Logger log = new Logger("StockHistoryImporter");
	StockTicker stockTicker;
	
	public StockHistoryImporter(StockTicker stockTicker){
		this.stockTicker = stockTicker;

	}
	public void getHistoryFromCsvFile(String ticker){
		try{
			FileReader fr = new FileReader("train\\"+ticker.toLowerCase()+".csv");
			BufferedReader br = new BufferedReader(fr);
			String fc = "";
			fc = br.readLine(); //frste linje med info
			fc = br.readLine(); //frste quote, som er den eldste
			double lastPrice = 0;
			while(fc != null) {
				//String dato = fc.substring(0,8);
				String[] tickerLine = fc.split(",");
				if(tickerLine.length==7){
					Calendar tickerDate = Calendar.getInstance();
					tickerDate.clear();
					
					String[] fullDate = tickerLine[0].split("-");
					int tickYear = Integer.parseInt(fullDate[2]);
					if(tickYear<100) tickYear = Integer.parseInt("19" + fullDate[2]);
					
					int tickDay = Integer.parseInt(fullDate[0]);
					
					int tickMonth = 0;
					
					String[] tickMonthTextValues = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
					for(int i=0;i<tickMonthTextValues.length;i++){
						if(tickMonthTextValues[i].equals(fullDate[1])){
							tickMonth = i;
						}
					}
					
					
																	           

					tickerDate.set(tickYear,tickMonth,tickDay);
					if(Double.parseDouble(tickerLine[3])>0){
						//Open 1 ,High 2,Low 3,Close 4,Volume 5
						stockTicker.addDay(tickerDate,Double.parseDouble(tickerLine[1]),Double.parseDouble(tickerLine[2]),Double.parseDouble(tickerLine[3]),Double.parseDouble(tickerLine[4]),Long.parseLong(tickerLine[5]),0.0);
						lastPrice = Double.parseDouble(tickerLine[4]);
					}
					else if(lastPrice>0){
						stockTicker.addDay(tickerDate,lastPrice,lastPrice,lastPrice,lastPrice,0,0.0);
					}
				}
				else log.warn("Did not import line from csv file");
				fc = br.readLine();
			}
				fr.close();
			
		} catch (IOException ioe){
			log.warn("skont i  henta meksefil");
			
		} catch (Exception e){
			log.warn("skontaexception:");
			 e.printStackTrace();
		}
		
		
		
		
	}
	
	
	public void getHistoryFromFile(String ticker){
		try{
			FileReader fr = new FileReader("historikk\\"+ticker.toLowerCase()+".txt");
			BufferedReader br = new BufferedReader(fr);
			String fc = "";
			fc = br.readLine(); //frste linje med info
			/*
				Del 0:20060929 (dato)
				Del 1:FOE (ticker)
				Del 2:Oslo Brs (brs)
				Del 3:267.00 (pning)
				Del 4:268.00 (high)
				Del 5:261.00 (low)
				Del 6:268.00 (close)
				Del 7:133580 (volum)
				Del 8:35507950 (verdi)
			*/
			fc = br.readLine(); //frste linje med info
			double lastPrice = 0;
			while(fc != null) {
				String[] tickerLine = fc.split("	");
				if(tickerLine.length==9){
					Calendar tickerDate = Calendar.getInstance();
					tickerDate.clear();

					String tickYear = tickerLine[0].substring(0,4);
					String tickMonth = tickerLine[0].substring(4,6);
					String tickDay = tickerLine[0].substring(6,8);
					int tickerMonth = Integer.parseInt(tickMonth);
					tickerMonth--; //date is 0-specified
					
					tickerDate.set(Integer.parseInt(tickYear),tickerMonth,Integer.parseInt(tickDay));
					if(Double.parseDouble(tickerLine[3])>0){
						stockTicker.addDay(tickerDate,Double.parseDouble(tickerLine[3]),Double.parseDouble(tickerLine[4]),Double.parseDouble(tickerLine[5]),Double.parseDouble(tickerLine[6]),Long.parseLong(tickerLine[7]),Double.parseDouble(tickerLine[8]));
						lastPrice = Double.parseDouble(tickerLine[6]);
					}
				}
				else log.warn("Did not import line from text file");
				fc = br.readLine();
			}
				fr.close();
			
		} catch (IOException ioe){
			log.warn("skont i  henta meksefil: " + ticker);
			
		} catch (Exception e){
			log.warn("skontaexception:");
			 e.printStackTrace();
		}
	}

	
	public void getHistoryFromNet(String ticker){
		log.debug("Loading from NET");
		try{
			URL page = new URL("http://hopey.netfonds.no/paperhistory.php?paper="+ticker.toUpperCase()+".OSE");
			InputStream is = page.openStream();
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			String fc = "";
			fc = br.readLine(); //reading not used line
			fc = br.readLine(); //reading first line
			while(fc != null) {
				String[] tickerLine = fc.split("	");
				if(tickerLine.length==9){
					Calendar tickerDate = Calendar.getInstance();
					tickerDate.clear();

					String tickYear = tickerLine[0].substring(0,4);
					String tickMonth = tickerLine[0].substring(4,6);
					String tickDay = tickerLine[0].substring(6,8);
					tickerDate.set(Integer.parseInt(tickYear),Integer.parseInt(tickMonth),Integer.parseInt(tickDay));
					if(Double.parseDouble(tickerLine[3])>0){
						stockTicker.addDay(tickerDate,Double.parseDouble(tickerLine[3]),Double.parseDouble(tickerLine[4]),Double.parseDouble(tickerLine[5]),Double.parseDouble(tickerLine[6]),Long.parseLong(tickerLine[7]),Double.parseDouble(tickerLine[8]));
					}
				}
				else log.warn("Did not import line from text file");
				fc = br.readLine();
			}
				is.close();
			
			
			
		}
		catch (Exception e){
			log.warn("Internet connection problem!");
			
		}
		

		
	}
	
	
	
	
	
	
}
