package no.ntnu.stockmeks;
import java.util.ArrayList;


import no.ntnu.stockmeks.agent.*;

/*
 * Main class. Starts different functinoality of the program
 */
public class StockTerminal {
	StockGui gui;
	Logger log = new Logger("StockTerminal");
	boolean addSuperAgents = false;

	public StockTerminal(){}
	

	/*
	 * Method returning available agents
	 */
	public ArrayList<Agent> getAgents(){
		ArrayList<Agent> agents = new ArrayList<Agent>();
		
		DoubleTopOrBottomAgent dtb = new DoubleTopOrBottomAgent();
		IslandReversalAgent ira = new IslandReversalAgent();
		MAAgent ma = new MAAgent();
		MaCrossoverAgent mac = new MaCrossoverAgent();
		PriceEnvelopeBandAgent peb = new PriceEnvelopeBandAgent(false);

		agents.add(dtb);
		agents.add(ira);
		agents.add(ma);
		agents.add(mac);
		agents.add(peb);
		
		GAPAgent breakawayGap = new GAPAgent(3,0.0,10);
		breakawayGap.setAgentName("Breakaway Gap");
		GAPAgent runawayGap = new GAPAgent(1,1.0,10);
		runawayGap.setAgentName("Runaway Gap");

		agents.add(breakawayGap);
		agents.add(runawayGap);

		RSIAgent rsi9days = new RSIAgent(9,75,25);
		RSIAgent rsi14days = new RSIAgent(14,71,29);
		RSIAgent rsi21days = new RSIAgent(21,70,30);
		agents.add(rsi9days);
		agents.add(rsi14days);
		agents.add(rsi21days);
		
		HeadAndShouldersAgent hs = new HeadAndShouldersAgent();
		agents.add(hs);
		
		MayAgent may = new MayAgent();
		agents.add(may);
		
		agents.add(new JanuaryAgent());
		agents.add(new RLandRHAgent());
		
		
		agents.add(new VolumeAgent());
		
		agents.add(new HangingManAgent());
		agents.add(new ReversalDayAgent());
		agents.add(new NeuralNetAgent());
		
		Agent[] agentskont = new Agent[agents.size()];
		agents.toArray(agentskont);
		
		
		MAAgent ma10_5Agent = new MAAgent(14,5);
		MAAgent ma200_10Agent = new MAAgent(200,10);
		
		agents.add(ma10_5Agent);
		agents.add(ma200_10Agent);

		
		
		Agent[] agentmeks = new Agent[agents.size()];
		agents.toArray(agentmeks);

		
		if(addSuperAgents){
			int[] classifierWeightsXtream = {2,-3,-1,-1,-10,8,9,-10,-10,-10,0,0,0,-1,1,-10,0,0}; //extreme
			int[] classifierWeights2 = {1,0,1,10,-1,0,0,-1,-1,0,0,0,10,0,1,-1,-10,0}; //mest korrekt
			int[] classifierWeightsBuySell = {0,3,2,-5,-2,-5,-2,9,10,0,-1,-10,1,-4,7,-10,-9,-2,2,-1};
			
			VotingClassifierAgent voter = new VotingClassifierAgent(agentskont,classifierWeights2,"Optimal");
			voter.setDays(2);

			VotingClassifierAgent voterX = new VotingClassifierAgent(agentskont,classifierWeightsXtream,"Extreme");
			voterX.setDays(2);

			VotingClassifierAgent voterBuySell = new VotingClassifierAgent(agentmeks,classifierWeightsBuySell,"Buy sell");
			voterBuySell.setDays(2);
			
			ID3TreeAgent id3 = new ID3TreeAgent(agentskont);
			J48TreeAgent j48 = new J48TreeAgent(agentskont);
			j48.setDays(3);
			agents.add(id3);
			agents.add(j48);

			agents.add(voterBuySell);
			agents.add(voter);
			agents.add(voterX);

			NeuralSuperAgent neuralSuper = new NeuralSuperAgent(agentmeks);
			agents.add(neuralSuper);
		}
		return agents;
	}
	
/*
	public StockTicker[] getTickers(){
		
		String[] tickerNames = {"foe","stl"};
		StockTicker[] tickers = new StockTicker[tickerNames.length];
		
		for(int i=0;i<tickers.length;i++){
			tickers[i] = new StockTicker(tickerNames[i]);
			tickers[i].doHistoryImport(tickers[i].IMPORT_FROM_FILE);
		}
		
		return tickers;
	}
*/	
	public StockTicker[] getTrainingTickers(){
		String[] tickerNames = {"ALU(Alcatel_Lucent)93-98","C(Citigroup)1apr87-1jul93","COP(Conoco_Phillips)86-99","DIS(Walt Disney)feb73-feb86","F(ford_motor)88-1mai94","FL(Foot_Locker)_71-86","GLW(Corning_Inc)Mar92-Mar96_sidelengs","HPQ(Hewlett_Packard)85-95","JNJ(Johnson_and_Johnson)82-89","PFE(Pfizer)84-91","PFE(Pfizer)May91-May95","RAD(Rite_Aid)92-98","S(Sprint)90-99","SLB(Schlumberger)May83-May97","XOM(Exxon_Mobile)1988-1997","bol-1dec83-1jun91","bol-1jan91-1jan99","cat97-1aug94","de-evikt-inov95","fpl-1mar85-99","GT-80-1may93","IBM-1aug79-1.mai79","mur-evikt-99","rsh-evigt-1aug97","tin-1apr90-99","WEN-1jun86-99"};
		StockTicker[] tickers = new StockTicker[tickerNames.length];
		
		for(int i=0;i<tickers.length;i++){
			tickers[i] = new StockTicker(tickerNames[i]);
			tickers[i].doHistoryImport(tickers[i].IMPORT_FROM_CSV);
		}
		return tickers;
	}
	
	public void getDecicionTreeTrainingSet(){
		addSuperAgents = false;
		ArrayList<Agent> agents = getAgents();
		StockTicker[] tickers = getTrainingTickers();
		
		DecisionTreeTestData treeData = new DecisionTreeTestData();
		treeData.generateTestData(agents,tickers,"train.arff");
	}
	

	/*
	 * Method evaluation the agents (voting classifier) performance over the parameterized years 
	 */
	public void startMonthlyEvaluator(int fromyear, int toyear){
		addSuperAgents = true;
		ArrayList<Agent> agents = getAgents();

		Agent[] agentmeks = new Agent[1];
		agentmeks[0] = agents.get(23);
		
		

		String[] tickerNames = getAllTickers();
		StockTicker[] tickers = new StockTicker[tickerNames.length];
		
		for(int i=0;i<tickers.length;i++){
			tickers[i] = new StockTicker(tickerNames[i]);
			tickers[i].doHistoryImport(tickers[i].IMPORT_FROM_FILE);
		}
		
		AgentEvaluator ae = new AgentEvaluator(tickers,agentmeks);
		ae.evaluateMonths(fromyear,toyear);
	}
	
	
	public String[] getWinningTickers(){
		String[] winTickers = {"AIK","ACTA","AKER","AKVER","AKY","ASC","BJORGE","BON","CRU-WIN","DNBNOR","DNO","DOF","EXPERT","FAR","FOE","FRO-WIN","GOGL","GRO","GRR","HEX","HNA","IGE-WIN","KOM","LSG","NAS","NHY","ODF","ODIM","ORK","PAN2","PAR","PRS","SCH-WIN","SCI","SDRL","SEVAN","SIOFF","SIT","SOFF","SRI","STL","SUB","SUO","TEL","TGS","TST","VEI","WWI","YAR"};
		return winTickers;
	}
	
	public String[] getLosingTickers(){
		String[] loseTickers = {"ACY-TAB","ALX-TAB","DOM","EME","KVE","NUT","PHO","TAD","TCO-TAB","ASD-TAB","CNR-TAB","COR-TAB","EDBASA-TAB","ELT-TAB","IGNIS-TAB","ITE-TAB","OPC-TAB","OTR-TAB","PAN1-TAB","SCH-TAB","SOI-TAB","STP-TAB","TOM-TAB"};
		return loseTickers;
		
	}
	public String[] getVolatileTickers(){
		String[] volatileTickers = {"APP","BIRD","CECO-VOL","CNR-VOL","FARA","FAST","GOL-VOL","KIT","KOG-VOL","MEC","NEC","NOD-VOL","NSG","NTL-VOL","OTR","PSI","QFR-VOL","RCL","REVUS","SAS-NOK-VOL","SNI","STB","TAT","TAA-VOL"};
		return volatileTickers;
	}
	public String[] getAllTickers(){
		String[] allTickers = {"ASC","ACY","ACTA","AKER","AKASA","AKD","AKFP","AKVER","AKS","AKY","AIK","ALX","APL","APP","AWO","ASD","GAS","BIOTEC","BIRD","BJORGE","BLO","BLU","BON","CECO","CNR","CAPTU","CEQ","COV","COR","CRU","DAT","DEEP","DESS","DIAG","DNBNOR","DNO","DOF","DOM","DYNA","EDRILL","EDBASA","EIOF","EKO","ELT","EME","EXPERT","FARA","FAR","FAST","FOE","FRO","FUNCOM","GRO","GEO","GGS","GOL","GOGL","GRR","GGG","HNA","HAVI","HEX","IGNIS","IMAREX","INM","IGE","ITE","JIN","KIT","KOM","KOG","KVE","LSG","MEC","NTL","NAVA","NOD","NEC","NSG","NHY","NAS","NUT","OCR","ODF","ODIM","OPERA","ORK","OTR","PAN1","PAN2","PAR","JACK","PHO","POLI","PRS","PSI","QFR","QEC","REVUS","RGT","RCL","SAS-NOK","SCI","SCH","SCORE","SDRL","SEVAN","SIOFF","SIT","SIN","SOI","SOFF","SONG","SRI","STL","STP","SNI","STB","SUB","SUO","TAA","TAD","TST","TAT","TCO","TEL","TGS","TOM","TREF","VEI","WWI","YAR"};
		return allTickers;
	}
	
	/*
	 * Method starting evaluation of agents performance
	 */
	public void startEvaluator(){
		addSuperAgents = true;
		ArrayList<Agent> agents = getAgents();

		Agent[] agentskont = new Agent[agents.size()];
		agents.toArray(agentskont);
		
		

		String[] tickerNames = getAllTickers();
		StockTicker[] tickers = new StockTicker[tickerNames.length];
		
		for(int i=0;i<tickers.length;i++){
			tickers[i] = new StockTicker(tickerNames[i]);
			tickers[i].doHistoryImport(tickers[i].IMPORT_FROM_FILE);
		}


		AgentEvaluator ae = new AgentEvaluator(tickers,agentskont);
		ae.evaluate(false,true,18);
	}
	
	
	/*
	 * Method starting the gui
	 */
	public void startGui(){
		addSuperAgents = true;
		gui = new StockGui(getAgents());
		gui.createAndShowGui();
	}

	/*
	 * Method starting genetic evolution with given population and eveolutions
	 */
	public void startGenetics(int population,int evolutions,boolean test,int mode){
		addSuperAgents = false;
		
		GeneticMain gm = new GeneticMain(population,evolutions,mode);
		ArrayList<Agent> agents = getAgents();
		
		StockTicker[] tickers;
		if(test){
			String[] tickerNames = {"foe","jin"};
			 tickers = new StockTicker[tickerNames.length];
			
			for(int i=0;i<tickers.length;i++){
				tickers[i] = new StockTicker(tickerNames[i]);
				tickers[i].doHistoryImport(tickers[i].IMPORT_FROM_FILE);
			}
		}
		else{
			tickers = getTrainingTickers();
		}
		
		
		
		Agent[] agenter = new Agent[agents.size()];
		for(int i=0;i<agents.size();i++){
			agenter[i] = agents.get(i);
		}

		
		try{
			gm.findSuperAgent(tickers,agenter);
		}
		catch (Exception e){
			log.error("Problems with GA!");
			e.printStackTrace();
		}
	}

	
	/*
	 * Main method. Invokes different functionality based on input arguments
	 * 
	 */
	public static void main(String[] args){
		StockTerminal st = new StockTerminal();
		
		if(args.length>0){
			String cmd = args[0];
			if(cmd.equals("genetics")){
				if(args.length<3) st.startGenetics(11,21,true,0);
				else{
					int population = Integer.parseInt(args[1]);
					int evolutions = Integer.parseInt(args[2]);
					st.startGenetics(population,evolutions,false,0);
				}
			}
			else if(cmd.equals("testgenetics")){
				if(args.length<3) st.startGenetics(11,21,true,0);
				else{
					int population = Integer.parseInt(args[1]);
					int evolutions = Integer.parseInt(args[2]);
					st.startGenetics(population,evolutions,true,0);
				}
			}
			else if(cmd.equals("geneticsextreme")){
				if(args.length<3) st.startGenetics(11,21,true,1);
				else{
					int population = Integer.parseInt(args[1]);
					int evolutions = Integer.parseInt(args[2]);
					st.startGenetics(population,evolutions,false,1);
				}
			}
			else if(cmd.equals("geneticsdays")){
				if(args.length<3) st.startGenetics(11,21,true,2);
				else{
					int population = Integer.parseInt(args[1]);
					int evolutions = Integer.parseInt(args[2]);
					st.startGenetics(population,evolutions,false,2);
				}
				
			}
			else if(cmd.equals("geneticsbuysell")){
				if(args.length<3) st.startGenetics(11,21,true,3);
				else{
					int population = Integer.parseInt(args[1]);
					int evolutions = Integer.parseInt(args[2]);
					st.startGenetics(population,evolutions,false,3);
				}
			}
			else if(cmd.equals("tree")) st.getDecicionTreeTrainingSet();
			else if(cmd.equals("evaluate")) st.startEvaluator();
			else if(cmd.equals("evaluatem")){
				if(args.length<3) st.startMonthlyEvaluator(1999,2006);
				else st.startMonthlyEvaluator(Integer.parseInt(args[1]),Integer.parseInt(args[2]));
			}
			else if(cmd.equals("train")) st.getDecicionTreeTrainingSet();
			else st.startGui();
		}
		else{
			st.startGui();
		}
	}
}
