package no.ntnu.stockmeks;

import java.util.Calendar;
import java.util.ArrayList;

/*
 * Class denoting a stock ticker history.
 * Has methods for importing and getting
 * different aspects of trade history
 */
public class StockTicker {
	public String ticker;
	public String exch;
	
	public double highest = 0.0;
	public double lowest = 0.0;

	public ArrayList<Calendar> dateList;
	public ArrayList<Double> openList;
	public ArrayList<Double> highList;
	public ArrayList<Double> lowList;
	public ArrayList<Double> closeList;
	public ArrayList<Long> volumeList;
	public ArrayList<Double> valueList;
	
	public final int IMPORT_FROM_FILE = 1;
	public final int IMPORT_FROM_NET = 2;
	public final int IMPORT_FROM_CSV = 3;

	Logger log = new Logger("StockTicker");
	StockHistoryImporter shi;
	
	public StockTicker(String ticker,String exch){
		this.ticker = ticker;
		this.exch = exch;
		dateList = new ArrayList<Calendar>();
		openList = new ArrayList<Double>();
		highList = new ArrayList<Double>();
		lowList = new ArrayList<Double>();
		closeList = new ArrayList<Double>();
		volumeList = new ArrayList<Long>();
		valueList = new ArrayList<Double>();
		shi = new StockHistoryImporter(this);

	}
	public StockTicker(String ticker){
		//log.debug("Making a new ticker-object with ticker: " + ticker);

		this.ticker = ticker;
		this.exch = "Oslo Brs";

		dateList = new ArrayList<Calendar>();
		openList = new ArrayList<Double>();
		highList = new ArrayList<Double>();
		lowList = new ArrayList<Double>();
		closeList = new ArrayList<Double>();
		volumeList = new ArrayList<Long>();
		valueList = new ArrayList<Double>();
		shi = new StockHistoryImporter(this);
	}
	public StockTicker(){
		log.debug("Making a new ticker-object");
		dateList = new ArrayList<Calendar>();
		openList = new ArrayList<Double>();
		highList = new ArrayList<Double>();
		lowList = new ArrayList<Double>();
		closeList = new ArrayList<Double>();
		volumeList = new ArrayList<Long>();
		valueList = new ArrayList<Double>();
		shi = new StockHistoryImporter(this);
	}
	
	public void setTicker(String ticker){
		log.debug("Setting a ticker name");
		this.ticker = ticker;
		clearHistory();
	}
	public void setExch(String exch){
		this.exch = exch;
	}
	public void clearHistory(){
		dateList.clear();
		openList.clear();
		highList.clear();
		lowList.clear();
		closeList.clear();
		volumeList.clear();
		valueList.clear();
		
	}
	
	public void doHistoryImport(int mode){
		clearHistory();
		if(mode==1){
			//loade fra fil
			shi.getHistoryFromFile(this.ticker);
		}
		else if(mode==2){
			//loade fra nettet
			shi.getHistoryFromNet(this.ticker);

		}
		else if(mode==3){
			shi.getHistoryFromCsvFile(this.ticker);
		}
	}
	
	public int getSize(){
		return closeList.size();
	}
	public int getSize(Calendar date){
		int dateStart = 0;
		for(int i=0;i<dateList.size()-1;i++){
			if((dateList.get(i)).before(date) && (dateList.get(i+1)).after(date)){
				dateStart = i;
			}
		}
		//m her hente alle datoer som er etter spesiell dato
		return (dateList.size()-dateStart);
	}
	
	public int getStart(Calendar date){
		int dateStart = 0;
		for(int i=0;i<dateList.size()-1;i++){
			if((dateList.get(i)).before(date) && (dateList.get(i+1)).after(date)){
				dateStart = i;
			}
		}
		return dateStart;
	}
	
	public String getTickerName(){
		return ticker;
	}
	
	public double getHigh(int i){
		return highList.get(i);
	}
	public double getLow(int i){
		return lowList.get(i);
	}
	public double getClose(int i){
		return closeList.get(i);
	}
	public double getOpen(int i){
		return openList.get(i);
	}
	
	public boolean isUpCloseDay(int i){
		if(i==0) return false;
		else if(closeList.get(i)>closeList.get(i-1)) return true;
		else return false;
	}
	
	public double getLowest(int from){
		//return getLowest(from,lowList.size(),lowList);
		return getLowest(from,lowList.size(),closeList);
	}
	public double getHighest(int from){
		//return getHighest(from,lowList.size(),highList);
		return getHighest(from,lowList.size(),closeList);
	}

	
	public double getLowest(int from, int to){
		return getLowest(from,to,lowList);
	}
	public double getLowestClose(int from,int to){
		return getLowest(from,to,closeList);
	}
	


	
	
	
	
	public double[] getExtremeDays(double percentValue){
		double[] extremeYield = new double[getSize()];
		
		for(int i=0;i<getSize();i++){
			double thisDay = getClose(i);
			double nextExtreme = getNextRelativeExtreme(i,percentValue);
			
			extremeYield[i] = nextExtreme/thisDay;
		}
		return extremeYield;
	}
	
	
	public double getNextRelativeExtreme(int from,double percentValue){
		//looking for next relative top or bottom from a specified day number
		
		int i=from+1;
		boolean foundExtreme = false;
		double highest = -1;
		int highestDay = -1;
		double lowest = -1;
		int lowestDay = -1;
		double extreme = 0;
		int extremeTurnIsAtDay = 0;
		int extremeDay = 0;
		double priceAtOriginDay = this.getClose(from);
		int maxDays = 365;

		
		
		while(!foundExtreme&& i<getSize() && i<(from+maxDays)){
			double atDay = getClose(i);
			
			if(atDay>highest){
				highest = atDay;
				highestDay = i;
			}
			if(atDay<lowest || lowest==-1){
				lowest = atDay;
				lowestDay = i;
			}
			
			//checking wether we are below or above bottom/top with a minimum nof percent
			double notBelow = highest*((100.00-percentValue)/(double)100.00);
			double notAbove = lowest*((100.00+percentValue)/(double)100.00);
			
			if(atDay>priceAtOriginDay){
				if(atDay<notBelow){
					extreme = highest;
					foundExtreme = true;
					extremeTurnIsAtDay = i;
					extremeDay = highestDay;
				}
			}
			else if(atDay<priceAtOriginDay){
				if(atDay>notAbove){
					extreme = lowest;
					foundExtreme = true;
					extremeTurnIsAtDay = i;
					extremeDay = lowestDay;
				}
			}
			
			
			i++;
		}
		
		double toReturn;
		if(!foundExtreme){
			if(getClose(getSize()-1)>priceAtOriginDay) toReturn = highest;
			else toReturn = lowest;
		}
		else{
			toReturn = extreme;
		}
		
		//log.debug(signal + " signal at day "+getDateString(from)+" (" + priceAtOriginDay+"): Extreme at " + getDateString(extremeDay) + ": " + toReturn);
		return toReturn;
	}
	
	public String getTextSignal(int atDay){
		double percentageHigher = evaluateSignal(atDay);
		
		if(percentageHigher>1.05) return "sell";
		else if(percentageHigher<0.95) return "buy";
		else return "hold";
	}
	public int getIntSignal(int atDay){
		double percentageHigher = evaluateSignal(atDay);
		
		if(percentageHigher>1.05) return -1;
		else if(percentageHigher<0.95) return 1;
		else return 0;
	}
	
	public double evaluateSignal(int atDay){
		double thisDay = getClose(atDay);
		double nextExtreme = getNextRelativeExtreme(atDay,5.0);
		double percentageHigher = thisDay / nextExtreme;
		return percentageHigher;
	}
	
	public String[] evaluateStringSignals(){
		String[] signals = new String[closeList.size()];
		for(int i=0;i<closeList.size();i++){
			signals[i] = getTextSignal(i);
		}
		return signals;
	}
	public int[] evaluateSignals(){
		int[] signals = new int[closeList.size()];
		for(int i=0;i<closeList.size();i++){
			signals[i] = getIntSignal(i);
		}
		return signals;
	}

	/*	
	public double getYield(int[] agentSignals, int startAt, int endAt){
		//int[] signals = new int[endAt-startAt];

		double roiBest = 0;
		
		for(int i=startAt;i<endAt;i++){
			double thisDay = getClose(i);
			double nextExtreme = getNextRelativeExtreme(i,5.0);
			
			if(agentSignals[i]==-1) roiBest += (-1) + (thisDay/nextExtreme);
			else if(agentSignals[i]==1) roiBest += (-1) + (nextExtreme/thisDay);
		}
		
		return roiBest;
	}
*/	
	
	public double getLowest(int from, int to,ArrayList<Double> quotelist){
		return quotelist.get(getLowestDay(from,to,quotelist));
	}
	
	public int getLowestDay(int from, int to,ArrayList<Double> quotelist){
		//finding lowest value form 
		if(from>to){
			int newTo = from;
			from = to;
			to = newTo;
		}
		else if(from==to) return from;

		if(from>=quotelist.size()){ 
			from = quotelist.size()-1;
		}
		if(to>=quotelist.size()){
			to = quotelist.size()-1;
		}
		

		double theLowest = quotelist.get(from);
		int lowestDay = from;
		for(int i=from;i<=to;i++){
			if(quotelist.get(i)<theLowest){
				theLowest = quotelist.get(i);
				lowestDay = i;
			}
		}
		return lowestDay;
	}

	public boolean isBefore(int atDay,int day,int month,int year){
		Calendar meks = dateList.get(atDay);
		Calendar skont = Calendar.getInstance();
		skont.clear();

		skont.set(year,(month-1),day);
		

		if(skont.getTimeInMillis()>meks.getTimeInMillis()) return true;
		else return false;
	}
	
	
	
	public boolean isAfter(int atDay,int day,int month,int year){
		Calendar meks = dateList.get(atDay);
		Calendar skont = Calendar.getInstance();
		skont.clear();
		
		skont.set(year,(month-1),day);
		

		//log.debug(getDateString(atDay) + " er etter " + day + "/" + month + "-" + year +": " +skont.after(meks));

		//log.debug("Kompared: " + skont.compareTo(meks));
		
		if(skont.getTimeInMillis()<meks.getTimeInMillis()) return true;
		else return false;

		//skont.set(year,(month-1),day);
		//if(skont.after(meks)) return true;
		//else return false;
	}
	
	public int getStartDayNumber(int from,int day,int month,int year){
		int theDay = -1;
		
		for(int i=from;i<getSize();i++){
			if(isAfter(i,day,month,year) && theDay==-1){
				theDay = i;
			}
		}
		
		return theDay;
	}

	public int getEndDayNumber(int day,int month,int year){
		int theDay = -1;
		
		for(int i=0;i<getSize();i++){
			if(isBefore(i,day,month,year)){
				theDay = i;
			}
		}
		return theDay;
	}

	
	
	public double getHighestClose(int from,int to){
		return getHighest(from,to,closeList);
	}
	public double getHighest(int from, int to){
		return getHighest(from,to,highList);
	}
	
	public int getHighestCloseDay(int from,int to){
		return getHighestDay(from,to,closeList);
	}
	public int getLowestCloseDay(int from,int to){
		return getLowestDay(from,to,closeList);
	}
	
	

	public int getHighestDay(int from,int to,ArrayList<Double> quotelist){
		if(from>to){
			int newTo = from;
			from = to;
			to = newTo;
		}
		else if(from==to) return from;

		if(from>=quotelist.size()){ 
			from = quotelist.size()-1;
		}
		if(to>=quotelist.size()){
			to = quotelist.size()-1;
		}


		double theHighest = 0.0;
		int highestDay = from;
		for(int i=from;i<=to;i++){
			if(quotelist.get(i)>theHighest){
				highestDay = i;
				theHighest = quotelist.get(i);
			}
		}
		return highestDay;
	}
	
	public double getHighest(int from,int to,ArrayList<Double> quotelist){
		return quotelist.get(getHighestDay(from,to,quotelist));
	}
	
	public long getVolume(int atDay){
		if(atDay>=getSize()) atDay = getSize()-1;
		else if(atDay<0) atDay = 0;
		return volumeList.get(atDay);
	}
	
	public int getMonth(int atDay){
		Calendar slafsa = dateList.get(atDay);
		return (slafsa.get(slafsa.MONTH)+1);
	}
	
	public int getYear(int atDay){
		Calendar meks = dateList.get(atDay);
		 
		return meks.get(meks.YEAR);
	}
	
	public String getDateString(int i){
		if(i>=dateList.size()) i = dateList.size()-1;
		Calendar meks = dateList.get(i);
		//String skont = meks.get(meks.DAY_OF_MONTH) + "/" + meks.get(meks.MONTH);
		int month = meks.get(meks.MONTH);
		int year = meks.get(meks.YEAR);
		month++;
		
		
		
		return meks.get(meks.DAY_OF_MONTH) + "/" + month + " " + year;
	}
	
	public String getDateString(int i,int from){
		if(i>=dateList.size()) i = dateList.size()-1;
		Calendar meks = dateList.get(dateList.size()-from+i);
		//String skont = meks.get(meks.DAY_OF_MONTH) + "/" + meks.get(meks.MONTH);
		
		return meks.get(meks.DAY_OF_MONTH) + "/" + (meks.get(meks.MONTH)+1) + " " + meks.get(meks.YEAR);
	}
	
	
	public void addDay(Calendar date,double open,double high, double low,double close,long volume,double value){
		//m parse datoen om til calendar-format
		//legger til arrayListen... skal dette gjres sortert tro?
		if(high>highest) highest = high;
		if(low<lowest || lowest==0) lowest = low;
		
//		if(dateList.size()>0){
//			if(((Calendar)dateList.get(0)).after(date)){
//				//legge til fr alle andre
//				dateList.add(0,date);
//				openList.add(0,new Double(open));
//				highList.add(0,new Double(high));
//				lowList.add(0,new Double(low));
//				closeList.add(0,new Double(close));
//				volumeList.add(0,new Long(volume));
//				if(value>0) valueList.add(0,new Double(value));
//			
//			}
//			else if(((Calendar)dateList.get(dateList.size()-1)).before(date)){
//				//legge til sist fordi denne er etter alle i lista
//				dateList.add(date);
//				openList.add(new Double(open));
//				highList.add(new Double(high));
//				lowList.add(new Double(low));
//				closeList.add(new Double(close));
//				volumeList.add(new Long(volume));
//				if(value>0) valueList.add(new Double(value));
//
//				
//			} 
//			else{
//				//mellom frst og sist, legger til der
//				for(int i=0;i<dateList.size()-1;i++){
//					if(((Calendar)dateList.get(i)).before(date) && ((Calendar)dateList.get(i+1)).after(date)){
//						//er bde fr og etter datoen som akkurat er skrevet inn. legger til
//						dateList.add((i+1),date);
//						openList.add((i+1),new Double(open));
//						highList.add((i+1),new Double(high));
//						lowList.add((i+1),new Double(low));
//						closeList.add((i+1),new Double(close));
//						volumeList.add((i+1),new Long(volume));
//						if(value>0) valueList.add((i+1),new Double(value));
//
//					}
//				}
//			}
//		}
		//else {
			//dette er frste ticker. legge til i starten av lista uansett
			dateList.add(0,date);
			openList.add(0,new Double(open));
			highList.add(0,new Double(high));
			lowList.add(0,new Double(low));
			closeList.add(0,new Double(close));
			volumeList.add(0,new Long(volume));
			if(value>0) valueList.add(0,new Double(value));

		//}
	}
	
	


}
