package no.ntnu.stockmeks;
import no.ntnu.stockmeks.agent.*;
import junit.framework.Assert;
import junit.framework.TestCase;


public class TestAgent extends TestCase {
	
	
	public void testGAP(){
		StockTicker ticker = new StockTicker("testGAP");
		ticker.doHistoryImport(ticker.IMPORT_FROM_FILE);

		Assert.assertEquals(271.59,ticker.getClose(20));
		Assert.assertEquals(271.59,ticker.getClose(21));
		Assert.assertEquals(471.59,ticker.getClose(22));
		
		GAPAgent commonGAP = new GAPAgent(1,0.0,2);
		GAPAgent breakawayGAP = new GAPAgent(2,5.0,10);
		GAPAgent breakawayGAPSimple = new GAPAgent(1,30.0,1);

		Assert.assertEquals(Agent.SIGNAL_ERROR,commonGAP.getSignal(ticker,1));
		Assert.assertEquals(Agent.SIGNAL_HOLD,commonGAP.getSignal(ticker,7));
		Assert.assertEquals(Agent.SIGNAL_BUY,commonGAP.getSignal(ticker,12));
		Assert.assertEquals(Agent.SIGNAL_BUY,commonGAP.getSignal(ticker,15));
		
		Assert.assertEquals(Agent.SIGNAL_ERROR,breakawayGAP.getSignal(ticker,7));
		Assert.assertEquals(Agent.SIGNAL_HOLD,breakawayGAP.getSignal(ticker,12));
		Assert.assertEquals(Agent.SIGNAL_BUY,breakawayGAP.getSignal(ticker,13));
		Assert.assertEquals(Agent.SIGNAL_SELL,breakawayGAP.getSignal(ticker,28));
		Assert.assertEquals(Agent.SIGNAL_HOLD,breakawayGAP.getSignal(ticker,30));

		Assert.assertEquals(Agent.SIGNAL_BUY,commonGAP.getSignal(ticker,23));
		Assert.assertEquals(Agent.SIGNAL_SELL,commonGAP.getSignal(ticker,28));
		
		Assert.assertEquals(Agent.SIGNAL_SELL,breakawayGAPSimple.getSignal(ticker,21));
		Assert.assertEquals(Agent.SIGNAL_BUY,breakawayGAPSimple.getSignal(ticker,30));

	}
	
	public void testRSI(){
		StockTicker ticker = new StockTicker("testRSI");
		ticker.doHistoryImport(ticker.IMPORT_FROM_FILE);
		RSIAgent rsiAgent = new RSIAgent(14);
		
		Assert.assertEquals(63.73626373626374,rsiAgent.calculateRSI(ticker,14,ticker.getSize()));
		Assert.assertEquals(57.55395683453237,rsiAgent.calculateRSI(ticker,21,ticker.getSize()));
	}
	
	public void testMA(){
		StockTicker ticker = new StockTicker("testMA");
		ticker.doHistoryImport(ticker.IMPORT_FROM_FILE);
		
		MAAgent maAgent = new MAAgent();
		maAgent.setNumDays(10);
		maAgent.setTicks(5);
		
		Assert.assertEquals(Agent.SIGNAL_BUY,maAgent.getSignal(ticker));
		Assert.assertEquals(Agent.SIGNAL_SELL,maAgent.getSignal(ticker,23));
		
		StockTicker tick2 = new StockTicker("tel");
		ticker.doHistoryImport(tick2.IMPORT_FROM_FILE);
		
		MAAgent maAgent2 = new MAAgent();
		maAgent2.setNumDays(187);
		maAgent2.setTicks(3);
		
		
	}
	
	public void testDoubleTopOrBottom(){
		StockTicker ticker = new StockTicker("testDoubleTB");
		ticker.doHistoryImport(ticker.IMPORT_FROM_FILE);
		
		DoubleTopOrBottomAgent dtb = new DoubleTopOrBottomAgent();

		Assert.assertEquals(Agent.SIGNAL_SELL,dtb.getSignal(ticker,5));
		Assert.assertEquals(Agent.SIGNAL_BUY,dtb.getSignal(ticker,10));
		Assert.assertEquals(Agent.SIGNAL_HOLD,dtb.getSignal(ticker,18));

	}
	
	
	
/*	
	public void testRSIsetOptimalValues(){
		StockTicker ticker = new StockTicker("tel");
		ticker.doHistoryImport(ticker.IMPORT_FROM_FILE);
		//RSIAgent rsiAgent = new RSIAgent(ticker,14);
		//rsiAgent.setOptimalValues();
		GAPAgent gap = new GAPAgent(ticker);
		gap.learnBestValues();
		
	}
*/	
}
