package no.ntnu.stockmeks.agent;
import no.ntnu.stockmeks.StockTicker;

public class DoubleTopOrBottomAgent implements Agent {

	double thresholdPercent = 10.0;
	double topPercent = 25.0;
	
	
	public int[] getSignalDays(StockTicker ticker) {
		return helper.getSignalDays(this,ticker);
	}

	
	
	public DoubleTopOrBottomAgent(){
	}
	
	public String getAgentName() {
		return "Double Top Bottom";
	}

	public int getSignal(StockTicker ticker) {
		return getSignal(ticker,ticker.getSize()-1);
	}

	public int getSignal(StockTicker ticker,int atDayNumber) {
		if(atDayNumber<3) return SIGNAL_ERROR;
		
		double lastClose;
		double thisClose;
		double thresholdValue;
		int first = -1;
		int second = -1;
		
		
		thisClose = ticker.getClose(atDayNumber);
		lastClose = ticker.getClose(atDayNumber-1);
		
		boolean checkingForDoubleTop = true;
		thresholdValue = thisClose * ((100.0-thresholdPercent)/100.0);
		
		if(thisClose>lastClose){
			checkingForDoubleTop = false;
			thresholdValue = thisClose * ((100.0+thresholdPercent)/100.0);
		}
		

		boolean secondExtremeRoundStarted = false;
		for(int i=atDayNumber-1;i>-1;i--){
			double theClose = ticker.getClose(i);
			
			if(checkingForDoubleTop){
				if(theClose<thresholdValue) break; //er dette den som skal mekse forlkka?
				else if(theClose<=thisClose){
					if(first<0){
						first = i;
					}
					else if(secondExtremeRoundStarted){
						second = i;
						break;
					}
				}
				else if(first>0 && theClose>thisClose){
					secondExtremeRoundStarted = true;
				}
				
			}
			else{
				//checking for double bottom
				//log.debug(atDayNumber + "_ thresholdvalue: " + thresholdValue);
				if(theClose>thresholdValue) break;
				else if(theClose>=thisClose){
					if(first<0) first = i;
					else if(secondExtremeRoundStarted){
						second = i;
						break;
					}
					
				}
				else if(first>0 && theClose<thisClose){
					secondExtremeRoundStarted = true;
				}

				
			}
		}
		double topLengthVariation = (double)((atDayNumber-first)/(double)(first-second));
		if(first<0 || second<0) return SIGNAL_HOLD;
		else if(topLengthVariation>5.0 || topLengthVariation<-5.0) return SIGNAL_HOLD; //hold when the signals are over a long time
		else{
			//checking tops between the two points
			if(checkingForDoubleTop){
				double minimumTop = thisClose * ((100.0+topPercent)/(double)100.0);
				double firstTop = ticker.getHighestClose(second,first);
				double secondTop = ticker.getHighestClose(first,atDayNumber);
				
				//first and second top is identified
				
				if(firstTop>minimumTop && secondTop>minimumTop){
					//both tops are above minimum level
					return SIGNAL_SELL;
				}
				else return SIGNAL_HOLD;
			}
			else{
				double maxBottom = thisClose * ((100.0-topPercent)/(double)100.0);
				double firstBottom = ticker.getLowestClose(second,first);
				double secondBottom = ticker.getLowestClose(first,atDayNumber);
				
				if(firstBottom<maxBottom && secondBottom<maxBottom){
					//both bottoms are below max bottom
					return SIGNAL_BUY;
				}
				else return SIGNAL_HOLD;
			}
		}
	}

	
	
	
}
