package no.ntnu.stockmeks.agent;
import no.ntnu.stockmeks.StockTicker;
public class GAPAgent implements Agent{
	int daysWithoutGapFilled = 3;
	double minimumGapPercent = 1.0;
	int daysPrecedingTheGap = 10;

	String agentName = "";
	
	public int[] getSignalDays(StockTicker ticker) {
		return helper.getSignalDays(this,ticker);
	}



	public GAPAgent(int daysWithoutGapFilled, double minimumGapPercent, int daysPrecedingTheGap){
		this.daysWithoutGapFilled = daysWithoutGapFilled;
		this.minimumGapPercent = minimumGapPercent;
		this.daysPrecedingTheGap = daysPrecedingTheGap;
	}
	

	
	public int getSignal(StockTicker ticker) {
		if(ticker.getSize()<daysPrecedingTheGap) return Agent.SIGNAL_ERROR;
		return getSignal(ticker,ticker.getSize()-1);
	}
	
	
	public boolean checkForUpGap(StockTicker ticker,int atDay){
		int checkDay = atDay - daysWithoutGapFilled;
		if(checkDay<1 || (checkDay-daysPrecedingTheGap)<0) return false;
		
		//log.debug("Checking for up gap at day: " + atDay);
		double highestLastDays = ticker.getHighest(checkDay-daysPrecedingTheGap,checkDay-1);
		
		boolean isGap = false;
		boolean gapIsFilled = false;
		
		if(ticker.getLow(checkDay)>highestLastDays*(1.00 + (minimumGapPercent/(double)100))){
			isGap = true;
			double requiredMinimumValue = highestLastDays*(1.00 + (minimumGapPercent/(double)100)); 
			
			for(int i=0;i<daysWithoutGapFilled;i++){
				if(ticker.getLow(checkDay+i+1)<requiredMinimumValue) gapIsFilled = true;
			}
		}
		
		
		if(isGap && !gapIsFilled) return true;
		else return false;
		
	}


	public boolean checkForDownGap(StockTicker ticker,int atDay){
		int checkDay = atDay - daysWithoutGapFilled;
		if(checkDay<1 || (checkDay-daysPrecedingTheGap)<0) return false;

		//log.debug("checking for down gap at day " + atDay + " with checkday at " + checkDay);

		double lowestLastDays = ticker.getLowest(checkDay-daysPrecedingTheGap,checkDay-1);
		boolean isGap = false;
		boolean gapIsFilled = false;
		
		if(ticker.getHigh(checkDay)<lowestLastDays*(1.00 - (minimumGapPercent/(double)100))){
			//log.debug("highest "+ticker.getDateString(checkDay)+" " + ticker.getHigh(checkDay) + " and lowest last days: " + lowestLastDays);
			isGap = true;
			double requiredMaximumValue = lowestLastDays*(1.00-(minimumGapPercent/(double)100));
			
			for(int i=0;i<daysWithoutGapFilled;i++){
				if(ticker.getHigh(checkDay+i+1)>requiredMaximumValue) gapIsFilled = true;
			}
		}
		
		if(isGap && !gapIsFilled) return true;
		else return false;
	}

	

	public int getSignal(StockTicker ticker,int atDayNumber) {
		if(atDayNumber<daysPrecedingTheGap) return Agent.SIGNAL_ERROR;

		if(checkForUpGap(ticker,atDayNumber)){
			return SIGNAL_BUY;
		}
		else if(checkForDownGap(ticker,atDayNumber)){
			return SIGNAL_SELL;
		}
		else return SIGNAL_HOLD;
	}


	public void setAgentName(String name){
		this.agentName = name;
	}

	public String getAgentName() {
		if(agentName.equals("")){
			return "GAP " + minimumGapPercent;
		}
		else return agentName;
	}



	public void setValues(int daysFilled,double percent,int daysPreceding){
		daysWithoutGapFilled = daysFilled;
		minimumGapPercent = percent;
		daysPrecedingTheGap = daysPreceding;
	}
	
	
	public void learnBestValues(StockTicker ticker){
		int optimalDaysWithoutGapFilled = daysWithoutGapFilled;
		double optimalMinimumGapPercent = minimumGapPercent;
		int optimalDaysPrecedingTheGap = daysPrecedingTheGap;
		double bestFitness = 0;
		
		for(int daysWithGap=0;daysWithGap<10;daysWithGap++){
			for(int daysPrecedingGap=0;daysPrecedingGap<50;daysPrecedingGap+=5){
				for(double gapPercent=0;gapPercent<20;gapPercent++){
					setValues(daysWithGap,gapPercent,daysPrecedingGap);
					double fitness = helper.getYield(ticker,this,0,ticker.getSize());

					if(fitness>bestFitness){
						bestFitness = fitness;
						optimalDaysPrecedingTheGap = daysPrecedingGap;
						optimalDaysWithoutGapFilled = daysWithGap;
						optimalMinimumGapPercent = gapPercent;
					}
				}
			}
			
		}
		
		log.debug("Satt nye gap-verdier: days preceding" + optimalDaysPrecedingTheGap + ", days without filled: " + optimalDaysWithoutGapFilled + ", percentage: " + optimalMinimumGapPercent + " (agent hadde fitnesss: " + bestFitness + ")");
		setValues(optimalDaysWithoutGapFilled,optimalMinimumGapPercent,optimalDaysPrecedingTheGap);
		
	}


	

}
