package no.ntnu.stockmeks.agent;

import no.ntnu.stockmeks.StockTicker;

public class HangingManAgent implements Agent{

	double solidBodyMinPart = 0.5;
	double maxHeadSize = 0.3;
	
	double maxUpperShadow = 0.05; 
	
	public String getAgentName() {
		return "Hanging man";
	}

	public int getSignal(StockTicker ticker) {
		return getSignal(ticker,ticker.getSize()-1);
	}

	public int getSignal(StockTicker ticker, int atDayNumber) {
		int dayForAdvance = atDayNumber - 10;
		if(identifyHangingMan(ticker,atDayNumber)){
			if(dayForAdvance<0) dayForAdvance = 0;
			
			if(ticker.getClose(dayForAdvance)>ticker.getClose(atDayNumber)){
				return SIGNAL_BUY;
			}
			else return SIGNAL_SELL;
		}
		else return SIGNAL_HOLD;
	}


	public int[] getSignalDays(StockTicker ticker) {
		return helper.getSignalDays(this,ticker);
	}
	
	public boolean identifyHangingMan(StockTicker ticker, int atDay){
		boolean isHangingMan = false;
		
		double lowest = ticker.getLow(atDay);
		double highest = ticker.getHigh(atDay);
		
		double open = ticker.getOpen(atDay);
		double close = ticker.getClose(atDay);
		
		double realBody = lowest-highest;
		double solidBody = open-close;

		if(realBody<0) realBody *= -1.0;
		if(solidBody<0) solidBody *= -1.0;
		
		double requiredHeadPlacement = Math.max(lowest,highest) - (realBody*(1.0-solidBodyMinPart));
		
		
		
		if(Math.min(open,close)>requiredHeadPlacement){
			double upperShadow = (Math.max(lowest,highest))-(Math.max(open,close));
			
			if((upperShadow/realBody)<maxUpperShadow){
				if((solidBody/realBody)<maxHeadSize){
					isHangingMan = true;
				}
			}
		}
		
		return isHangingMan;
	}

}
