package no.ntnu.stockmeks.agent;

import no.ntnu.stockmeks.StockTicker;

/*
 * Agent identifying head and shoulders patterns
 * 
 */
public class HeadAndShouldersAgent implements Agent{

	double neckPercent =  20;
	double headPercent = 40;
	double precedingAdvancePercent = 10;
	double precedingDeclinePercent = 5;
	
	double neckLinePercent = 3;
	
	public String getAgentName() {
		return "Head Shoulders";
	}

	public int getSignal(StockTicker ticker) {
		return getSignal(ticker,ticker.getSize()-1);
	}

	public int getSignal(StockTicker ticker, int atDayNumber) {
		boolean isHS = checkForHeadAndShoulder(ticker,atDayNumber);
		boolean isOHS = checkForReversedHeadAndShoulder(ticker,atDayNumber);
		if(isHS) return SIGNAL_SELL;
		else if(isOHS) return SIGNAL_BUY;
		else return SIGNAL_HOLD;
	}


	public int[] getSignalDays(StockTicker ticker) {
		return helper.getSignalDays(this,ticker);
	}
	
	
	public boolean checkForReversedHeadAndShoulder(StockTicker ticker, int atDay){
		double firstNeck = -1;
		int firstNeckAtDay = -1;
		int firstFirstNeckAtDay = -1;

		double lastNeck = -1;
		int lastNeckAtDay = -1;
		int lastFirstNeckAtDay = -1;
		
		double head = -1;
		int headAtDay = -1;
		int headFirstAtDay = -1;
				
		double priceAtDay = ticker.getClose(atDay);
		
		double neckValueMinimum = priceAtDay * ((100.00-neckPercent)/(double)100.00); 
		double neckLineMinimum = priceAtDay * ((100.00-neckLinePercent)/(double)100.00);
		double headMinimum = priceAtDay * ((100.00-headPercent)/(double)100.00); 
		double advanceMinimum = neckValueMinimum * ((100.00+precedingAdvancePercent)/(double)100.00);
		double advanceMaximum = neckValueMinimum * ((100.00-precedingDeclinePercent)/(double)100.00);

		//log.debug(priceAtDay + "_ neckMin: " + neckValueMinimum + ", headmin: " + headMinimum + ", advanceMin: " + advanceMinimum + ", advMax " +advanceMaximum);

		
		boolean allTopsFound = false;
		boolean formationCompleted = false;
		boolean firstTopOver = false;
		boolean headOver = false;
		boolean formationFailed = false;
		
		for(int i=atDay-1;i>0 && !formationCompleted;i--){
			double thisDay = ticker.getClose(i);
			if(!firstTopOver && thisDay<neckValueMinimum){
				if(firstFirstNeckAtDay==-1) firstFirstNeckAtDay = i;
				firstNeck = thisDay;
				firstNeckAtDay = i;
			}
			else if(firstNeck>0 && thisDay>neckLineMinimum && !firstTopOver){
				firstTopOver = true;
			}
			else if(!headOver && firstTopOver && thisDay<headMinimum){
				if(headFirstAtDay==-1) headFirstAtDay = i;
				head = thisDay;
				headAtDay = i;
			}
			else if(head>0 && !headOver && thisDay>neckLineMinimum){
				headOver = true;
			}
			else if(headOver && thisDay<neckValueMinimum && headOver){
				if(lastFirstNeckAtDay==-1) lastFirstNeckAtDay = i;
				lastNeck = thisDay;
				lastNeckAtDay = i;
			}
			else if(lastNeck>0 && thisDay>neckLineMinimum && !allTopsFound){
				allTopsFound = true;
				//log.debug("all reverse tops found... now looking for " +advanceMinimum + "as compared to " + thisDay);
				if(thisDay>advanceMinimum){
					formationCompleted = true;
				}
			}
			else if(allTopsFound && thisDay<advanceMaximum){
				formationCompleted = true;
				formationFailed = true;
			}
		}
		
		if(formationCompleted && !formationFailed){
			
			double neck1bottom = ticker.getHighestClose(lastFirstNeckAtDay,headAtDay);
			double neck2bottom = ticker.getHighestClose(headFirstAtDay,firstNeckAtDay);
			double neck1 = ticker.getLowestClose(lastNeckAtDay,lastFirstNeckAtDay);
			double headPeak = ticker.getLowestClose(headAtDay,headFirstAtDay);
			double neck2 = ticker.getLowestClose(firstNeckAtDay,firstFirstNeckAtDay);
			
/*
			if(atDay==27){
				log.debug(atDay + "_" + lastNeckAtDay + " " + lastFirstNeckAtDay);
				log.debug(atDay + "_" + headAtDay + " " + headFirstAtDay);
				log.debug(atDay + "_" + firstNeckAtDay + " " + firstFirstNeckAtDay);
				
				log.debug(atDay +": " + neck1bottom + "->" + neck1+ "->" + headPeak+ "->" + neck2bottom + "->" + neck2+ "->" +priceAtDay);
				
			}
			
*/			
			if(headPeak>=neck1 || headPeak>=neck2){
				return false;
			}
			else if(priceAtDay>neck2bottom){
				//a rising formation
				if(neck2bottom>neck1bottom){
					return true; //log.debug(atDay +"_ rising head an shoulders formation: " +priceAtDay+ "->" + lowestBetweenHeadAndLastShoulder + "->" + lowestBetweenFirstShoulderAndHead);
				}
				else return false;
			}
			else if(priceAtDay<neck2bottom){
				if(neck2bottom>neck1bottom){
					return true; //log.debug(atDay +"_ declining head an shoulders formation: " +priceAtDay+ "->" + lowestBetweenHeadAndLastShoulder + "->" + lowestBetweenFirstShoulderAndHead);
				}
				else return false;
			}
			else {
				if(neck1bottom==neck2bottom) return true;
				else return false;
			}
		}
		else return false; 
		
		
		
	}
	
	
	public boolean checkForHeadAndShoulder(StockTicker ticker,int atDay){
		
		double firstNeck = -1;
		int firstNeckAtDay = -1;
		int firstFirstNeckAtDay = -1;

		double lastNeck = -1;
		int lastNeckAtDay = -1;
		int lastFirstNeckAtDay = -1;
		
		double head = -1;
		int headAtDay = -1;
		int headFirstAtDay = -1;
				
		double priceAtDay = ticker.getClose(atDay);
		
		double neckValueMinimum = priceAtDay * ((100.00+neckPercent)/(double)100.00); 
		double neckLineMinimum = priceAtDay * ((100.00+neckLinePercent)/(double)100.00);
		double headMinimum = priceAtDay * ((100.00+headPercent)/(double)100.00); 
		double advanceMinimum = neckValueMinimum * ((100.00-precedingAdvancePercent)/(double)100.00);
		double advanceMaximum = neckValueMinimum * ((100.00+precedingDeclinePercent)/(double)100.00);
		
		
		
		boolean allTopsFound = false;
		boolean formationCompleted = false;
		boolean firstTopOver = false;
		boolean headOver = false;
		boolean formationFailed = false;
		
		for(int i=atDay-1;i>0 && !formationCompleted;i--){
			double thisDay = ticker.getClose(i);
			if(!firstTopOver && thisDay>neckValueMinimum){
				if(firstFirstNeckAtDay==-1) firstFirstNeckAtDay = i;
				firstNeck = thisDay;
				firstNeckAtDay = i;
			}
			else if(firstNeck>0 && thisDay<neckLineMinimum && !firstTopOver){
				firstTopOver = true;
			}
			else if(!headOver && firstTopOver && thisDay>headMinimum){
				if(headFirstAtDay==-1) headFirstAtDay = i;
				head = thisDay;
				headAtDay = i;
			}
			else if(head>0 && !headOver && thisDay<neckLineMinimum){
				headOver = true;
			}
			else if(headOver && thisDay>neckValueMinimum && headOver){
				if(lastFirstNeckAtDay==-1) lastFirstNeckAtDay = i;
				lastNeck = thisDay;
				lastNeckAtDay = i;
			}
			else if(lastNeck>0 && thisDay<neckLineMinimum && !allTopsFound){
				allTopsFound = true;
				if(thisDay<advanceMinimum){
					formationCompleted = true;
				}
			}
			else if(allTopsFound && thisDay>advanceMaximum){
				formationCompleted = true;
				formationFailed = true;
			}
		}
		
		if(formationCompleted && !formationFailed){
			
			double neck1bottom = ticker.getLowestClose(lastFirstNeckAtDay,headAtDay);
			double neck2bottom = ticker.getLowestClose(headFirstAtDay,firstNeckAtDay);
			double neck1 = ticker.getHighestClose(lastNeckAtDay,lastFirstNeckAtDay);
			double headPeak = ticker.getHighestClose(headAtDay,headFirstAtDay);
			double neck2 = ticker.getHighestClose(firstNeckAtDay,firstFirstNeckAtDay);
							
			if(headPeak<=neck1 || headPeak<=neck2){
				return false;
			}
			else if(priceAtDay>neck2bottom){
				//a rising formation
				if(neck2bottom>neck1bottom){
					return true; //log.debug(atDay +"_ rising head an shoulders formation: " +priceAtDay+ "->" + lowestBetweenHeadAndLastShoulder + "->" + lowestBetweenFirstShoulderAndHead);
				}
				else return false;
			}
			else if(priceAtDay<neck2bottom){
				if(neck2bottom>neck1bottom){
					//log.debug("finished: " + ticker.getDateString(atDay) + ",: " +priceAtDay);
					return true; //log.debug(atDay +"_ declining head an shoulders formation: " +priceAtDay+ "->" + lowestBetweenHeadAndLastShoulder + "->" + lowestBetweenFirstShoulderAndHead);
				}
				else return false;
			}
			else {
				if(neck1bottom==neck2bottom) return true;
				else return false;
			}
		}
		else return false; 
	}
	

}
