package no.ntnu.stockmeks.agent;
import no.ntnu.stockmeks.StockTicker;

public class MAAgent implements Agent{
	int numDays = 40;
	int numSuccedingTicks = 10;
	
	public MAAgent(){}
	public MAAgent(int numDays,int numTicks){
		this.numDays = numDays;
		this.numSuccedingTicks = numTicks;
	}

	

	
	
	public int[] getSignalDays(StockTicker ticker) {
		return helper.getSignalDays(this,ticker);
	}

		
	
	public String getAgentName() {
		return "Moving Average " + numDays + "_" + numSuccedingTicks + "";
	}

	public int getSignal(StockTicker ticker) {
		return getSignal(ticker,ticker.getSize()-1);
	}
	
	

	public int getSignal(StockTicker ticker,int atDayNumber) {
		if(atDayNumber>ticker.getSize() || atDayNumber<numDays) return SIGNAL_ERROR;

		int upDay = 0;
		int downDay = 0;
		double lastMa = 0;
		
		
		for(int i=(atDayNumber-numDays+1);i<=atDayNumber;i++){
			double thisMa = calculateMa(ticker,i);
			if(lastMa>0){
				if(lastMa>thisMa) downDay++;
				else upDay++;
				
			}
			lastMa = thisMa;
		}
		
		if(upDay==0) return SIGNAL_SELL;
		else if(downDay==0) return SIGNAL_BUY;
		else return SIGNAL_HOLD;
	}
	
	
	public double calculateMa(StockTicker ticker,int atDay){
		if((atDay-numDays)<0 || atDay==0) return 0;
		
		double totalPrice = 0;
		
		for(int i=(atDay-numDays+1);i<=atDay;i++){
			totalPrice += ticker.getClose(i);
		}
		return (totalPrice/numDays);
	}


	
	public void setNumDays(int days){
		this.numDays = days;
	}
	public void setTicks(int ticks){
		this.numSuccedingTicks = ticks;
	}
	
	public void setValues(int days, int ticks){
		this.numDays = days;
		this.numSuccedingTicks = ticks;
	}

	public void learnOptimalValues(StockTicker ticker,boolean showTransactions){
		int optimalDays = numDays;
		int optimalSuccedingTicksMin = numSuccedingTicks;
		int optimalSuccedingTicksMax = numSuccedingTicks;
		double bestFitness = 0;
		
		for(int days=5;days<200;days+=(int)(days*1.5)){
			for(int ticks=3;(ticks<(30)) && (ticks<days);ticks+=(int)(days/(5))){
				setValues(days,ticks);
				double fitness;
				if(showTransactions) fitness = helper.simulate(ticker,this,true);
				else fitness = helper.simulate(ticker,this);
				if(fitness>bestFitness){
					bestFitness = fitness;
					optimalDays = days;
					optimalSuccedingTicksMin = ticks;
					optimalSuccedingTicksMax = ticks;
				}
				else if(fitness==bestFitness){
					optimalSuccedingTicksMax = ticks;
				}
			}
		}
		
		int middleTicks = ((optimalSuccedingTicksMax+optimalSuccedingTicksMin)/2);
		
		setValues(optimalDays,middleTicks);
		log.debug("Optimal values set: days: " + optimalDays + ", and ticks: " + middleTicks + ", fitness: " + bestFitness);
		
	}
	public void learnOptimalValues(StockTicker ticker){
		learnOptimalValues(ticker,false);
	}
	

}
