package no.ntnu.stockmeks.agent;

import no.ntnu.stockmeks.StockTicker;

public class NeuralSuperAgent implements Agent {

	double[] nnetGood = {-10944.25,0.29,10943.32,-0.67,0.35,-0.71,0.78,1.04,0.96,0.54,-0.32,10944.74,-10944.14,10944.20,-0.25,10943.04,0.05,0.78,0.65,-0.26,-10943.88,-5994.20,1841.07,-1607.96,745.38,-331.67,2807.95,-208.21,14081.39,691.18,-37.55,1304.28,-3077.42,-3633.02,-1175.28,-9127.03,-389.33,22.22,-2339.72,1005.61,724.76,-2104.80,9229.09,-174.77,141.83,4509.50,-221.09,906.73,961.35,-10894.66,-647.10,-255.03,-2133.72,-10541.31,15331.91,313.35,2658.32,-2011.25,1803.86,185.95,-6349.38,1915.05,19521.34,-0.75,1.05,0.53,0.84};
	double[] nnetBad = {40.14,81.39,45.77,6.76,0.74,56.09,86.43,120.45,19.41,5.32,-0.73,14.51,3207.72,6417.61,-9.32,17.75,26.39,3243.67,-11.13,3228.32,3185.03,-11728.90,-438.71,794.93,2453.97,-71.25,-2439.30,7707.72,-4619.72,347.67,-8048.51,-1099.28,6100.29,-541.32,1356.57,-2608.72,765.13,-443.84,-7565.15,9365.93,-7925.72,10428.65,-77.37,-2913.44,3069.95,2969.08,29.34,-2628.41,-105.13,5223.70,-23.70,-693.20,-257.70,420.61,-6147.94,7868.72,-3219.63,3045.52,-41.66,-14639.59,-49.00,-8987.70,-2636.12,0.41,-0.55,0.40,-0.55};
	
	int netSize = 3;
	Agent[] agents;
	
	public NeuralSuperAgent(Agent[] agents){
		this.agents = agents;
	}
	
	public String getAgentName() {
		return "Neural Superagent";
	}

	public int getSignal(StockTicker ticker) {
		return getSignal(ticker,ticker.getSize()-1);
	}

	public int getSignal(StockTicker ticker, int atDayNumber) {

		double[] signals = new double[agents.length];
		for(int i=0;i<agents.length;i++){
			signals[i] = agents[i].getSignal(ticker,atDayNumber);
		}
		
		double goodEvaluation = evaluateNeural(signals,nnetGood);
		double badEvaluation = evaluateNeural(signals,nnetBad);
		
		if(goodEvaluation>0 && badEvaluation>0) return SIGNAL_BUY;
		else if(goodEvaluation<0 && badEvaluation<0) return SIGNAL_SELL;
		else return SIGNAL_HOLD;
	}


	public int[] getSignalDays(StockTicker ticker) {
		return helper.getSignalDays(this,ticker);
	}
	
	
	public double evaluateNeural(double[] data, double[] nNet){
		int nofPerceptrons = ((nNet.length-(netSize+1))/(netSize)) - 1; //total number of perceptrons minus the output
		
		double perceptronValue[] = new double[netSize];
		
		for(int inputToPerceptron=0;inputToPerceptron<netSize;inputToPerceptron++){
			perceptronValue[inputToPerceptron] = nNet[inputToPerceptron*nofPerceptrons];
			
			for(int i=1;i<nofPerceptrons;i++){
				perceptronValue[inputToPerceptron] += (nNet[i+(nofPerceptrons*inputToPerceptron)] * data[(i-1)]);
			}
			if(perceptronValue[inputToPerceptron]>0) perceptronValue[inputToPerceptron] = 1;
			else perceptronValue[inputToPerceptron] = -1;
		}
		// har satt alle perceptron-verdier
		
		double finalValue = nNet[nNet.length-(netSize+1)];
		for(int i=0;i<netSize;i++){
			finalValue += (nNet[nNet.length-netSize+i] * perceptronValue[i]);
		}
		return finalValue;
	}

}
