package no.ntnu.stockmeks.agent;
import no.ntnu.stockmeks.StockTicker;

public class PriceEnvelopeBandAgent implements Agent{

	double percentageValue = 3;
	double thresholdPercent = 5; 
	MAAgent ma;
	boolean useBollinger = false;

	public int[] getSignalDays(StockTicker ticker) {
		return helper.getSignalDays(this,ticker);
	}

	
	
	public PriceEnvelopeBandAgent(boolean useBollinger){
		this.ma = new MAAgent();
		this.useBollinger = useBollinger;
	}
	
	
	public String getAgentName() {
		return "Price Envelope Band";
	}

	public int getSignal(StockTicker ticker) {
		return getSignal(ticker,ticker.getSize()-1);
	}

	public int getSignal(StockTicker ticker,int atDayNumber) {
		if(atDayNumber>ticker.getSize()) return SIGNAL_ERROR;
		
		double movingAverage = ma.calculateMa(ticker,atDayNumber);
		if(movingAverage==0) return SIGNAL_ERROR;

		double resistance;
		double support;
		resistance = movingAverage*((percentageValue+100.0)/(double)100.0);
		support= movingAverage*((100.0-percentageValue)/(double)100.0);

		
		double threshold = (resistance-support) * (thresholdPercent/(double)100.0); 
		
		double stockPrice = ticker.getClose(atDayNumber);
		if(stockPrice>(resistance-threshold)) return SIGNAL_SELL;
		else if(stockPrice<(support+threshold)) return SIGNAL_BUY;
		else return SIGNAL_HOLD;
	}

	
	
	public void setValues(double percentage, double threshold){
		this.percentageValue = percentage;
		this.thresholdPercent = threshold;
	}
	
	public void learnOptimalValues(StockTicker ticker,boolean showTransactions){
		double optimalPercentage = percentageValue;
		double optimalThreshold = thresholdPercent;
		double optimalThresholdMax = thresholdPercent;
		double bestFitness = 0;
		
		for(double percent=0.5;percent<20;percent+=0.5){
			for(double threshold=0;threshold<20;threshold++){
				setValues(percent,threshold);
				double fitness = helper.simulate(ticker,this,showTransactions);
				if(fitness>bestFitness){
					bestFitness = fitness;
					optimalPercentage = percent;
					optimalThreshold = threshold;
					optimalThresholdMax = threshold;
				}
				else if(fitness==bestFitness){
					optimalThresholdMax = threshold;
					
				}
			}
		}
		
		double threshAvg = ((optimalThreshold+optimalThresholdMax)/2.0);
		
		setValues(optimalPercentage,threshAvg);
		log.debug("Optimal values set: percentage: " + optimalPercentage + ", and threshold: " + threshAvg + ", fitness: " + bestFitness);
	}
	
	
	

}
