package no.ntnu.stockmeks.agent;

import no.ntnu.stockmeks.StockTicker;
import java.util.ArrayList;


/*
 * Trend agent. Evaluationg relative high and lows
 */
public class RLandRHAgent implements Agent{

	double thresholdPercent = 30.0;
	
	String hasGoneHunted = "";
	
	double significantDifference = 0.1;
	
	ArrayList<Integer> lowList = new ArrayList<Integer>();
	ArrayList<Integer> highList = new ArrayList<Integer>();

	
	public String getAgentName() {
		return "Trend";
	}

	public int getSignal(StockTicker ticker) {
		return getSignal(ticker,ticker.getSize()-1);
	}

	public int getSignal(StockTicker ticker, int atDayNumber) {
		if(!hasGoneHunted.equals(ticker.getTickerName())){
			setRLAndRH(ticker);
		}
				

		int foundHighAtDay = -1;
		for(int i=0;i<highList.size();i++){
			if(highList.get(i)<=atDayNumber) foundHighAtDay = i;
		}

		int foundLowAtDay = -1;
		for(int i=0;i<lowList.size();i++){
			if(lowList.get(i)<=atDayNumber) foundLowAtDay = i;
		}
		
		//log.debug("found low at day: " + foundLowAtDay + ", high at " + foundHighAtDay);
		
		if(foundLowAtDay<0 || foundHighAtDay<0) return SIGNAL_ERROR;
		
		double youWereThelastHigh = ticker.getClose(highList.get(foundHighAtDay)); 
		double haveYouEverBeenThisLow = ticker.getClose(lowList.get(foundLowAtDay)); 
		
		double highTrend = 0;
		double lowTrend = 0;
		
		
		for(int i=foundHighAtDay;i>0;i--){
			double previousHigh = ticker.getClose(highList.get(i));
			if((previousHigh*(1.00+significantDifference))<youWereThelastHigh){
				//highTrend = youWereThelastHigh / (youWereThelastHigh-previousHigh);
				highTrend = 1;
				i=-1;
			}
			else if(previousHigh*(1.00-significantDifference)>youWereThelastHigh){
				highTrend = -1;
				i=-1;
			}
		}

		for(int i=foundLowAtDay;i>0;i--){
			double previousLow = ticker.getClose(highList.get(i));
			
			if(previousLow*(1.00+significantDifference)<haveYouEverBeenThisLow){
				lowTrend = 1;
				i=-1;
			}
			else if(previousLow*(1.00-significantDifference)>haveYouEverBeenThisLow){
				lowTrend = -1;
				i=-1;
			}
		}
		
		
		double trend = highTrend + lowTrend;
		
		if(trend<-1) return SIGNAL_SELL;
		else if(trend>1) return SIGNAL_BUY;
		else return SIGNAL_HOLD;

		
	}


	public int[] getSignalDays(StockTicker ticker) {
		return helper.getSignalDays(this,ticker);
	}
	
	
	public void setRLAndRH(StockTicker ticker){
		highList.clear();
		lowList.clear();
		
		double lastHigh = ticker.getClose(0);
		double lastLow = lastHigh;
		int lastLowest = 0;
		int lastHighest = 0;
		
		
		
		for(int i=1;i<ticker.getSize();i++){
			double thisDay = ticker.getClose(i);
			
			if(thisDay>lastHigh){
				lastHigh = thisDay;
				lastHighest = i;
				
				if(thisDay>(lastLow*((100.00+thresholdPercent)/100.00))){
					lowList.add((lastLowest));
					lastLow = thisDay;
				}
			}
			else if(thisDay<lastLow){
				lastLow = thisDay;
				lastLowest = i;
				
				if(thisDay<(lastHigh*((100.00+thresholdPercent)/100.00))){
					highList.add(lastHighest);
					lastHigh = thisDay;
					
				}
				
			}

			hasGoneHunted = ticker.getTickerName();			
			
			
		}
		
		
		
		
	}

	
	
	
	
	
	
	
	
	
	
	
	
	
	public void goHuntingHighAndLow(StockTicker ticker){
		double firstDay = ticker.getClose(0);
		
		double lastHigh = firstDay;
		double lastLow = firstDay;
		double lastRH = firstDay;
		double lastRL = firstDay;
		int lastRHAt = 0;
		int lastRLAt = 0;
				
		lowList.clear();
		highList.clear();
		
		
		for(int i=1;i<ticker.getSize();i++){
			double thisDay = ticker.getClose(i);
			if(thisDay>lastHigh){
				lastHigh = thisDay;
				if(thisDay>lastRH){
					lastRHAt = i;
					lastRH = thisDay;
				}
				if(thisDay>(lastRL*(100.00+thresholdPercent/(100.00)))){
					lowList.add(lastRLAt);
					lastRL = thisDay;
					lastLow = thisDay;
					//log.debug("Adding skont");

				}
			}
			else if(thisDay<lastLow){
				lastLow = thisDay;
				lastRLAt = i;
				if(thisDay<lastRL){
					lastRLAt = i;
					lastRL = thisDay;
				}

				if(thisDay<(lastRH*(100.00-thresholdPercent/(100.00)))){
					highList.add(lastRHAt);
					lastRH = thisDay;
					lastHigh = thisDay;
					//log.debug("Adding meks");
				}
			}
		}
		
		hasGoneHunted = ticker.getTickerName();
	
	}
	
	

}
