package no.ntnu.stockmeks.agent;

import no.ntnu.stockmeks.StockTicker;

public class RSIAgent implements Agent{
	int numDays = 14; //default number of days
	int thresholdOversold = 30;
	int thresholdOverbought = 70;
	
	String optimalTicker = "";

	boolean doSetOptimalValues = false;
	
	public int[] getSignalDays(StockTicker ticker) {
		return helper.getSignalDays(this,ticker);
	}

	
	public RSIAgent(){
		doSetOptimalValues = true;
	}


	public RSIAgent(int numDays){
		this.numDays = numDays;
	}
	public RSIAgent(int numDays, int overbought, int oversold){
		this.numDays = numDays;
		this.thresholdOverbought = overbought;
		this.thresholdOversold = oversold;
	}
	
	
	public int getSignal(StockTicker ticker) {
		if(ticker.getSize()<numDays) return Agent.SIGNAL_ERROR;
		return getSignal(ticker,ticker.getSize()-1);
	}

	public int getSignal(StockTicker ticker,int atDayNumber){
		if(doSetOptimalValues && !optimalTicker.equals(ticker.getTickerName())){
			optimalTicker = ticker.getTickerName();
			setOptimalValues(ticker);
		}

		
		if(atDayNumber<=numDays) return SIGNAL_ERROR;
		

		
		double rsi = calculateRSI(ticker,numDays, atDayNumber);
		if(rsi==-1) return SIGNAL_ERROR;
		else if(rsi<thresholdOversold) return SIGNAL_BUY;
		else if(rsi>thresholdOverbought) return SIGNAL_SELL;
		else return SIGNAL_HOLD;
	}
	
	

	
	
	
	public double calculateRSI(StockTicker ticker,int days, int atDayNumber){
		if(days>ticker.getSize()) days = ticker.getSize();
		double totalGainForUpCloses = 0;
		double totalLossForDownCloses = 0;

		for(int i=atDayNumber-days;i<ticker.getSize() && i<atDayNumber;i++){
			if(ticker.isUpCloseDay(i)) totalGainForUpCloses += (ticker.getClose(i) - ticker.getClose(i-1));
			else totalLossForDownCloses += (ticker.getClose(i-1) - ticker.getClose(i));
		}
		return (100 - (100/(1 + (totalGainForUpCloses/days)/(totalLossForDownCloses/days))));
	}

	public String getAgentName() {
		if(doSetOptimalValues) return "Predictive RSI";
		else return "RSI " + numDays + " days";
	}


	public void setOptimalValues(StockTicker ticker){
		setOptimalValues(ticker,0,ticker.getSize());
	}
	
	public void setOptimalValues(StockTicker ticker,int fromDay, int toDay){
		int optimalDays = numDays;
		int optimalSold = thresholdOversold;
		int optimalBought = thresholdOverbought;
		double bestfitness = 0;
		
		
		
		for(int days=7;days<30;days++){
			for(int threshSold=15;threshSold<40;threshSold+=5){
				for(int threshBought=60;threshBought<95;threshBought+=5){
					numDays=days;
					thresholdOverbought=threshBought;
					thresholdOversold=threshSold;
					
					double fitness = helper.getYield(ticker,this,fromDay,toDay);
					
					
					if(bestfitness<fitness){
						optimalDays = days;
						optimalSold = threshSold;
						optimalBought = threshBought;
						bestfitness = fitness;
					}
				}
			}
		}
		numDays = optimalDays;
		thresholdOversold = optimalSold;
		thresholdOverbought = optimalBought;

		
		
	}

	
}
